
#
# Generic-GNU-arm-ra6m3_m13 specific settings
#

# Board description
SET(CMAKE_BOARD_MANUFACTURER Renesas)
SET(CMAKE_BOARD_NAME RA6M3_M13)
SET(CMAKE_BOARD_OS FreeRTOS10)

#------------------------------------------------------------------------------
# 1. define compiler and linker settings
# Set C++ compiler flags
SET(CMAKE_CXX_FLAGS_DEBUG   "-mcpu=cortex-m4 -mthumb -mfloat-abi=hard -mfpu=fpv4-sp-d16 -Og -fmessage-length=0 -fsigned-char -ffunction-sections -fdata-sections -fno-inline-functions -Wpointer-arith -Wlogical-op  -g -D_RENESAS_RA_ -D_DEBUG")
SET(CMAKE_CXX_FLAGS_RELEASE   "-mcpu=cortex-m4 -mthumb -mfloat-abi=hard -mfpu=fpv4-sp-d16 -O2 -fmessage-length=0 -fsigned-char -ffunction-sections -fdata-sections -fno-inline-functions -Wpointer-arith -Wlogical-op -D_RENESAS_RA_")

SET(CMAKE_CXX_FLAGS_DEBUGFIXPOINT   "${CMAKE_CXX_FLAGS_DEBUG}")
SET(CMAKE_CXX_FLAGS_RELEASEFIXPOINT "${CMAKE_CXX_FLAGS_RELEASE}")

# Set C compiler flags
SET(CMAKE_C_FLAGS_DEBUG           "${CMAKE_CXX_FLAGS_DEBUG}")
SET(CMAKE_C_FLAGS_RELEASE         "${CMAKE_CXX_FLAGS_RELEASE}")
SET(CMAKE_C_FLAGS_DEBUGFIXPOINT   "${CMAKE_CXX_FLAGS_DEBUGFIXPOINT}")
SET(CMAKE_C_FLAGS_RELEASEFIXPOINT "${CMAKE_CXX_FLAGS_RELEASEFIXPOINT}")

# Add subdirectory for BSP
IF(STREAMRUNTIME_SDK_PROJECT)
    SET(CMAKE_BSP_SUBDIRECTORY "${PROJECT_SOURCE_DIR}/../Renesas/BSP")
ELSE(STREAMRUNTIME_SDK_PROJECT)
    SET(CMAKE_BSP_SUBDIRECTORY "${PATH_BSP}/BSP")
ENDIF(STREAMRUNTIME_SDK_PROJECT)

# Linker flags
SET(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -Wl,-M=${PROJECT_NAME}.map -Wl,--start-group -Wl,--end-group -Xlinker --gc-sections -L${CMAKE_BSP_SUBDIRECTORY}/script -Wl,--whole-archive -lBSP -Wl,--no-whole-archive")

# Include directories
TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra/arm/CMSIS_5/CMSIS/Core/Include"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra/aws/FreeRTOS/FreeRTOS/Source/include"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra/fsp/inc"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra/fsp/inc/api"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra/fsp/inc/instances"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra/fsp/src/bsp/cmsis/Device/RENESAS/Include"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra/fsp/src/bsp/mcu/all"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra/fsp/src/bsp/mcu/ra6m3"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra/fsp/src/rm_freertos_port"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra/fsp/src/r_drw"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra/tes/dave2d/inc"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra_cfg/aws"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra_cfg/driver"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra_cfg/fsp_cfg"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra_cfg/fsp_cfg/bsp"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/ra_gen"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/src"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/src/m13design"
  PRIVATE "${CMAKE_BSP_SUBDIRECTORY}/src/m13design/font"
)

#------------------------------------------------------------------------------
# 2. define preprocessor defines
# Set common defines (e.g. "eC_TARGET_COMPILER_GCC; eC_TARGET_ENV_LINUX")
SET(COMMON_DEFINES "eC_TARGET_COMPILER_GCC;eC_TARGET_ENV_FREERTOS;eC_TARGET_CPU_RA6M3;eC_TARGET_CPU_ARM_NEON;PLATFORM_RENESAS_RA6M3;GUILIANI_TASK_STACK_SIZE=32768" CACHE INTERNAL "common defines" FORCE)

# Linker Script
SET(LINKER_SCRIPT "-T fsp.ld")

#------------------------------------------------------------------------------
# 4. define global stuff
TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
  PRIVATE "${PATH_EGAC}/comps/eC/code/FreeRTOS"
  PRIVATE "${PATH_GUILIANI}/Include/Platform/general"
)

IF(LIBS_BUILD_FREETYPE)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_FREETYPE}/include" 
  )
ELSE(LIBS_BUILD_FREETYPE)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_LIBS}/freetype/include"
    PRIVATE "${PATH_LIBS}/freetype/include/freetype" 
  )
  TARGET_LINK_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_LIBS}/freetype/lib"
  )
ENDIF(LIBS_BUILD_FREETYPE)

IF(STREAMRUNTIME_SDK_PROJECT)
ELSE(STREAMRUNTIME_SDK_PROJECT)
  IF(EGML_NEON_OPTIMIZATION)
  #------------------------------------------------------------------------------
  # 4. define global stuff
  INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_EGAC}/comps/eGML/code/eGML_NEON"
    PRIVATE "${PATH_EGAC}/comps/eGML/code/eGML_NEON/easy_crc"
  )
  ENDIF(EGML_NEON_OPTIMIZATION)
ENDIF(STREAMRUNTIME_SDK_PROJECT)

#------------------------------------------------------------------------------
# 5. define executable file extension
set(CMAKE_EXECUTABLE_SUFFIX ".x")
