/*
 *#######################################################################
 * Copyright (C) 2012-2013 TES Electronic Solutions GmbH,
 * All Rights Reserved.
 * This source code and any compilation or derivative thereof is the
 * proprietary information of TES Electronic Solutions GmbH
 * and is confidential in nature.
 * Under no circumstances is this software to be exposed to or placed
 * under an Open Source License of any type without the expressed
 * written permission of TES Electronic Solutions GmbH
 *
 *#######################################################################
 */

#ifndef __GUI_FONT_H__
#define __GUI_FONT_H__

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

#include "gui_config.h"
#include "gui_types.h"

/**
 A wrapper struct ("class") for glyphlib 4bpp bitmap fonts
 */
typedef struct gui_font_s
{
    void * font_handle; /* library-internal "glyphlib" handle */
} gui_font_t;

/**
 Describes basic font attributes
 */
typedef struct gui_font_metrics_s
{
    tfix16 line_height; /**< advance y between two lines of text (1:9:6 fixed point) */
    tfix16 ascender; /**< space above baseline (1:9:6 fixed point)                   */
    tfix16 descender; /**< space below baseline (1:9:6 fixed point)                  */
    tfix16 y_ppem;
    tfix16 x_ppem;
    tfix16 em_units;
} gui_font_metrics_t;

/**
 @brief Initialize font

 @param font        Font which will be initialized. Used as "this" parameter.
 @param data        Data
 @param dataSize    Data size

 @return true if initialization was successful, false otherwise.
 */
GUI_EXTERN tbool GUI_CALL gui_font_init(
    gui_font_t* font,
    const void* data,
    tu32 dataSize
);

/**
 @brief De-Initialize font

 @param font        Font which will be de-initialized. Used as "this" parameter.
 */
GUI_EXTERN void GUI_CALL gui_font_deinit(gui_font_t* font);

/**
 @brief Calculate advance width for a single line of text. Returns a 1:25:6 fixed point value.

 @param font        Font, used as "this" parameter.
 @param text        Text

 @return Text width
 */
GUI_EXTERN ts32 GUI_CALL gui_font_get_string_advance_width(
    const gui_font_t* font,
    const tchar* text
);

/**
 @brief Query font height, ascender, descender. Please see gui_font_metrics_t.

 @param font            Font, used as "this" parameter
 @param retMetrics      Metrics
 */
GUI_EXTERN void GUI_CALL gui_font_get_metrics(
    const gui_font_t* font,
    gui_font_metrics_t* retMetrics
);

/**
 @brief Query if font is compressed (A4)

 @param font            Font, used as "this" parameter

 @return true if font is compressed
 */
GUI_EXTERN tbool GUI_CALL gui_font_is_compressed(const gui_font_t* font);

/**
 @brief Layout a single line of text. Return the baseline position in pos.
 Set layoutFlags to GUI_LAYOUT_ADAPT[XY] to adjust sizeAvail to the text extents

 @param font            Font
 @param text            Text
 @param retPos          Position
 @param sizeAvail       Available size
 @param layoutFlags     Layout flags
 */
GUI_EXTERN void GUI_CALL gui_font_layout_string(
    const gui_font_t* font,
    const tchar* text,
    gui_pos16_t* retPos,
    gui_size16_t* sizeAvail,
    tu16 layoutFlags
);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __GUI_FONT_H__ */
