/*
 *#######################################################################
 * Copyright (C) 2012-2013 TES Electronic Solutions GmbH,
 * All Rights Reserved.
 * This source code and any compilation or derivative thereof is the
 * proprietary information of TES Electronic Solutions GmbH
 * and is confidential in nature.
 * Under no circumstances is this software to be exposed to or placed
 * under an Open Source License of any type without the expressed
 * written permission of TES Electronic Solutions GmbH
 *
 *#######################################################################
 */

#ifndef __GUI_TYPES_H__
#define __GUI_TYPES_H__

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

/** GUI bool type*/
#define GUI_TRUE  1
#define GUI_FALSE 0

typedef unsigned int targb32;

#ifdef GUI_USE_BLU_TYPES
/* Define GUI types using BLU types */

typedef u8 tu8;
typedef s8 ts8;
typedef u16 tu16;
typedef s16 ts16;
typedef u32 tu32;
typedef s32 ts32;
typedef sbool tbool;
typedef sfix16 tfix; /* 1:15:16 */

typedef blu_pos16_t gui_pos16_t;
typedef blu_size16_t gui_size16_t;
typedef blu_geo16_t gui_geo16_t;

#else
/* Define GUI types using standard types*/

typedef unsigned char tu8;
typedef signed char ts8;
typedef unsigned short tu16;
typedef signed short ts16;
typedef unsigned int tu32;
typedef signed int ts32;
typedef signed int tbool;
typedef signed int tfix; /* 1:15:16 */

typedef struct gui_pos16_s
{
    ts16 x;
    ts16 y;
} gui_pos16_t;

typedef struct gui_size16_s
{
    ts16 x;
    ts16 y;
} gui_size16_t;

typedef struct gui_geo16_s
{
    gui_pos16_t p;
    gui_size16_t s;
} gui_geo16_t;

#endif /* GUI_USE_BLU_TYPES */

typedef ts16 tfix16; /* 1:9:6 fixed point */

typedef char tchar8;
/* #ifdef _WIN32 */
/* typedef wchar_t        tchar16; */
/* #endif */
typedef tu16 tchar16;

#ifdef GUI_TCHAR_UNICODE_16
typedef tchar16 tchar;
#else
typedef tchar8 tchar;
#endif /* GUI_TCHAR_UNICODE_16 */

typedef tu32 tsize;

typedef union gui_memptr_u_
{
    void *any;
    tu8 *ptr8;
    tu16 *ptr16;
    tu32 *ptr32;
    union
    {
        tu32 u32;
    } val;

    union
    {
        const void *any;
        const tu8 *ptr8;
        const tu16 *ptr16;
        const tu32 *ptr32;
    } ro;
} gui_memptr_u;

typedef struct gui_pos8_s
{
    ts8 x;
    ts8 y;
} gui_pos8_t;

typedef struct gui_size8_s
{
    tu8 x;
    tu8 y;
} gui_size8_t;

typedef struct gui_geo8_s
{
    gui_pos8_t p; /* Position */
    gui_size8_t s; /* Size */
} gui_geo8_t;

GUI_EXTERN const gui_pos16_t gui_pos16_0;
GUI_EXTERN const gui_size16_t gui_size16_0;

/**
 Alignment flags
 */
#define GUI_LAYOUT_NONE    0u
#define GUI_LAYOUT_LEFT    1u
#define GUI_LAYOUT_RIGHT   2u
#define GUI_LAYOUT_ADAPTX  4u
#define GUI_LAYOUT_CENTERX (GUI_LAYOUT_LEFT | GUI_LAYOUT_RIGHT)
#define GUI_LAYOUT_XMASK   (GUI_LAYOUT_LEFT | GUI_LAYOUT_RIGHT | GUI_LAYOUT_ADAPTX)

#define GUI_LAYOUT_TOP     1024u
#define GUI_LAYOUT_BOTTOM  2048u
#define GUI_LAYOUT_ADAPTY  4096u
#define GUI_LAYOUT_CENTERY (GUI_LAYOUT_TOP | GUI_LAYOUT_BOTTOM)
#define GUI_LAYOUT_YMASK   (GUI_LAYOUT_TOP | GUI_LAYOUT_BOTTOM | GUI_LAYOUT_ADAPTY)

#define GUI_LAYOUT_CENTER  (GUI_LAYOUT_CENTERX | GUI_LAYOUT_CENTERY)
#define GUI_LAYOUT_ADAPT   (GUI_LAYOUT_ADAPTX | GUI_LAYOUT_ADAPTY)

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __GUI_TYPES_H__ */
