
#
# Windows-MSVC-AMD64 settings file
#

# set output dirs
SET_TARGET_PROPERTIES(${PROJECT_NAME} PROPERTIES LIBRARY_OUTPUT_DIRECTORY "${PROJECT_SOURCE_DIR}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}")
SET_TARGET_PROPERTIES(${PROJECT_NAME} PROPERTIES ARCHIVE_OUTPUT_DIRECTORY "${PROJECT_SOURCE_DIR}/Libraries/${CMAKE_SYSTEM_NAME}-${CMAKE_SYSTEM_ARCH}/${CMAKE_CXX_COMPILER_ID}")

TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
  PRIVATE "${PATH_GUILIANI}/Include/Platform/win"
  PRIVATE "${PATH_GUILIANI}/Resources/win/pc"
)

# BLU
IF(GUILIANI_ADD_BLU_PLATFORM)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Platform/BLU"
    PRIVATE "${PATH_BLU}/Include"
  )
ENDIF()

# OpenGL
IF(GUILIANI_ADD_GL_PLATFORM)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_LIBS}/opengl/include"
  )
ENDIF()

# Vulkan
IF(GUILIANI_ADD_VULKAN_PLATFORM)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Platform/Vulkan"
    PRIVATE "${PATH_VULKAN}/Include"
  )
ENDIF()

# platform-dependencies
IF(GUILIANI_ADD_IMAGE_DECODER)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_LIBS}/gif"
    PRIVATE "${PATH_LIBS}/nanosvg/include"
  )
ENDIF()

# FREETYPE
IF(LIBS_BUILD_FREETYPE)
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_FREETYPE}/include" 
  )
ELSE()
  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_LIBS}/freetype/include" 
    PRIVATE "${PATH_LIBS}/freetype/include/freetype" 
  )
ENDIF()

# image-libs
IF(NOT EGAC_BUILD_IMAGELIBS)
  IF(LIBS_BUILD_LIBJPEG)
    TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_LIBJPEG}"
    )
  ELSE()
    TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_LIBS}/jpeg"
    )
  ENDIF()

  IF(LIBS_BUILD_LIBPNG)
    TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_LIBPNG}"
    )
  ELSE()
    TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_LIBS}/png/lpng1636"
    )
  ENDIF()

  # if zlib is built include-file is located in build-folder
  IF(NOT LIBS_BUILD_ZLIB)
    TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_LIBS}/zlib/include"
    )
  ENDIF()
ENDIF()

FILE(GLOB GUILIANI_PLATFORM_WIN_CPP "${PROJECT_SOURCE_DIR}/Source/Platform/win/*.cpp" )
SOURCE_GROUP("Source Files\\Platform\\Windows" FILES ${GUILIANI_PLATFORM_WIN_CPP})
FILE(GLOB GUILIANI_PLATFORM_WIN_H "${PROJECT_SOURCE_DIR}/Include/Platform/win/*.h" )
SOURCE_GROUP("Header Files\\Platform\\Windows" FILES ${GUILIANI_PLATFORM_WIN_H})

# DAVE
IF(GUILIANI_ADD_DAVE_PLATFORM)
  SET(GUILIANI_PLATFORM_DAVE_CPP
    "${PROJECT_SOURCE_DIR}/Source/Platform/DAVE/GfxEnvDAVE.cpp"
  )
  SET(GUILIANI_PLATFORM_DAVE_H
    "${PROJECT_SOURCE_DIR}/Include/Platform/DAVE/GfxEnvDAVE.h"
  )

  IF(STREAMRUNTIME_SDK_PROJECT)
    TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_DAVE}/Include"
      PRIVATE "${PATH_DAVE}/Include/d0lib"
      PRIVATE "${PATH_DAVE}/Include/softdave"
    )
  ELSE()
    TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
      PRIVATE "${PATH_GUILIANI}/../DAVE/Include"
      PRIVATE "${PATH_GUILIANI}/../DAVE/Include/d0lib"
      PRIVATE "${PATH_GUILIANI}/../DAVE/Include/softdave"
    )
  ENDIF()
ENDIF()

# DAVE HD
IF(GUILIANI_ADD_DAVEHD_PLATFORM)
  FILE(GLOB GUILIANI_PLATFORM_DAVEHD_WIN_CPP "${PROJECT_SOURCE_DIR}/Source/Platform/DAVEHD/win/*.cpp" )
  SOURCE_GROUP("Source Files\\Platform\\DAVEHD\\win" FILES ${GUILIANI_PLATFORM_DAVEHD_WIN_CPP})
  FILE(GLOB GUILIANI_PLATFORM_DAVEHD_WIN_H "${PROJECT_SOURCE_DIR}/Include/Platform/DAVEHD/win/*.h" )
  SOURCE_GROUP("Header Files\\Platform\\DAVEHD\\win" FILES ${GUILIANI_PLATFORM_DAVEHD_WIN_H})

  TARGET_INCLUDE_DIRECTORIES(${PROJECT_NAME}
    PRIVATE "${PATH_GUILIANI}/Include/Platform/DAVEHD/win"
    PRIVATE "${PATH_LIBS}/DAVEHD/inc/driver/config"
    PRIVATE "${PATH_LIBS}/DAVEHD/inc/driver/config/platform"
    PRIVATE "${PATH_LIBS}/DAVEHD/inc/driver/kernel"
    PRIVATE "${PATH_LIBS}/DAVEHD/inc/driver/user"
    PRIVATE "${PATH_LIBS}/DAVEHD/inc/softdave"
  )
ENDIF()

SET(GUILIANI_PLATFORM_FILES 
  ${GUILIANI_PLATFORM_WIN_CPP}
  ${GUILIANI_PLATFORM_WIN_H}
  ${GUILIANI_PLATFORM_DAVE_CPP}
  ${GUILIANI_PLATFORM_DAVE_H}
  ${GUILIANI_PLATFORM_DAVEHD_WIN_CPP}
  ${GUILIANI_PLATFORM_DAVEHD_WIN_H}
)
