/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GUI_CMDBEHAVIOUR__H_
#define GUI_CMDBEHAVIOUR__H_

#include "GUIBehaviourDecorator.h"
#include "GUICommand.h"

/** Base class which Associates a command with one event slot. 
The event will be forwarded to the associated object.
@see CGUICmdBehaviour, CGUIMultiCmdBehaviour
*/

class CGUICmdBehaviour : public CGUIBehaviourDecorator
{
protected:
    /**
        Constructor.
    */
    CGUICmdBehaviour() {}

    /**
        Destructor.
    */
    virtual ~CGUICmdBehaviour() {}

    virtual eC_Bool DoClick(
        const eC_Value& vAbsX = eC_FromInt(-1),
        const eC_Value& vAbsY = eC_FromInt(-1));

    virtual eC_Bool DoLongClick(
        const eC_Value& vAbsX = eC_FromInt(-1),
        const eC_Value& vAbsY = eC_FromInt(-1));

    virtual eC_Bool DoDoubleClick(
        const eC_Value& vAbsX = eC_FromInt(-1),
        const eC_Value& vAbsY = eC_FromInt(-1));

    virtual eC_Bool DoButtonDown(
        const eC_Value& vAbsX,
        const eC_Value& vAbsY);

    virtual eC_Bool DoButtonUp(
        const eC_Value& vAbsX,
        const eC_Value& vAbsY);

    virtual eC_Bool FocusPrevious(
        CGUIObject* const pRefObj = NULL,
        const eC_Bool &bSubTreeOfRefObjDone = false);

    virtual eC_Bool FocusNext(
        CGUIObject* const pRefObj = NULL,
        const eC_Bool &bSubTreeOfRefObjDone = false);

    virtual eC_Bool Decrease();

    virtual eC_Bool Increase();

    virtual eC_Bool DoDrag(
        const eC_Value& vDeltaX,
        const eC_Value& vDeltaY,
        const eC_Value& vAbsX,
        const eC_Value& vAbsY);

    virtual eC_Bool DoDragEnd(
        const eC_Value& vAbsX,
        const eC_Value& vAbsY);

    virtual eC_Bool DoMouseEnter(
        const eC_Value& vAbsX,
        const eC_Value& vAbsY);

    virtual eC_Bool DoMouseLeave(
        const eC_Value& vAbsX,
        const eC_Value& vAbsY);

    virtual eC_Bool DoScroll(
        const eC_Value& vAbsX,
        const eC_Value& vAbsY,
        const eC_Value& vDelta,
        const eC_Bool& bModifierActive);

    virtual eC_Bool DoKeyDown(
        const GUIKeyIdentifier_t& eKeyID,
        const eC_UInt& uiModifiers);

    virtual eC_Bool DoKeyUp(
        const GUIKeyIdentifier_t& eKeyID,
        const eC_UInt& uiModifiers);

    virtual eC_Bool DoChar(
        const eC_UInt& uiKey,
        const GUIKeyIdentifier_t& eKeyID,
        const eC_UInt& uiModifiers);

    virtual eC_Bool DoScale(const eC_Value& vDelta);

    virtual void GetFocus();

    virtual void LoseFocus();

    virtual void OnCreate();

    virtual void OnDelete();

    virtual void OnShow();

    virtual void OnHide();

    virtual void OnDataPool(const DataPoolResource_t& eID);

    /**
        @brief Executes the command for the given behaviour, if it is equal to the stored one.
        @param eFromBehaviour Behaviour slot from which this method was called.
    */
    virtual void ExecuteCommand(const BehaviourType_t& eFromBehaviour) = 0;
};
#endif
