/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef GUISINGLECMDBEHAVIOUR__H_
#define GUISINGLECMDBEHAVIOUR__H_

#include "GUICmdBehaviour.h"

/**
    This class gives the possibility to execute a command when a specific
    event (e.g. click event, drag event, etc.) occurs.
    @brief Execute a command when a specific event occurs.
*/
class CGUISingleCmdBehaviour : public CGUICmdBehaviour
{
public:
    /**
        Constructor.
    */
    CGUISingleCmdBehaviour();

    /**
        Destructor.
    */
    virtual ~CGUISingleCmdBehaviour() {}

    /**
        Sets the command for a specific behaviour.
        @param pkCommand The command to execute.
        @param eBehaviourType The behaviour type.
    */
    void SetCmdForBehaviour(
        CGUICommandPtr pkCommand,
        const BehaviourType_t& eBehaviourType);

    virtual CGUIBehaviourDecorator* Clone() const;

#ifdef GUILIANI_STREAM_GUI
    virtual void ReadFromStream();
#endif
#ifdef GUILIANI_WRITE_GUI
    virtual void WriteToStream(const eC_Bool bWriteClassID=false);
#endif

#if defined(GUILIANI_STREAM_GUI) || defined(GUILIANI_WRITE_GUI)
    /// XML tag to be used when writing a behaviour type into a stream.
    static const eC_Char XMLTAG_BEHAVIOURTYPE[];
#endif

protected:
    /**
        Executes the command for the given behaviour, if it is equal
        to the stored one.
        @param eFromBehaviour Behaviour slot from which this method was called.
    */
    virtual void ExecuteCommand(const BehaviourType_t& eFromBehaviour);

private:
    /** Copy-constructor.
        @param kSource Source object to be copied.
    */
    CGUISingleCmdBehaviour(const CGUISingleCmdBehaviour& kSource);

    /** Operator= method.
        @param kSource Source object to be copied.
        @return This instance.
    */
    CGUISingleCmdBehaviour& operator=(const CGUISingleCmdBehaviour& kSource);

private:
    /// Stores the behaviour type.
    BehaviourType_t m_eBehaviourType;

    /// Stores the command for the behaviour;
    CGUICommandPtr m_pkCommand;
};
#endif
