#ifndef GUI_STREAM_TYPES_H_
#define GUI_STREAM_TYPES_H_

#include "eC_Types.h"

/// File type for Streams
enum FileType_t
{
    FILE_TYPE_XML = 0,    ///< XML
    FILE_TYPE_BINARY = 1, ///< Binary
    FILE_TYPE_CUSTOM = 2  ///< User defined custom type
};

/// File endianess for streams
enum FileEndianess_t
{
    FILE_ENDIANESS_BIG = 0,    ///< big endian
    FILE_ENDIANESS_LITTLE = 1, ///< little endian
    FILE_ENDIANESS_INVALID = 255   ///< Invalid/not applicable. E.g. for XML
};

/// File alignment for Streams
enum FileAlignment_t
{
    FILE_ALIGNMENT_8BIT = 0,  ///< byte aligned
    FILE_ALIGNMENT_16BIT = 1, ///< 16-bit aligned
    FILE_ALIGNMENT_32BIT = 2, ///< 32-bit aligned
    FILE_ALIGNMENT_64BIT = 3, ///< 64-bit aligned
    FILE_ALIGNMENT_INVALID = 255  ///< Invalid/not applicable. E.g. for XML
};

// File structure is 0xEEAAVVVV (E = Endianess, A = Alignment, V= Version
const eC_UInt cSHIFT_ENDIANESS = 24; ///< shift value for endianess in binary streaming header
const eC_UInt cSHIFT_ALIGNMENT = 16; ///< shift value for alignment in binary streaming header
const eC_UInt cSHIFT_VERSION = 0;    ///< shift value for version in binary streaming header

const eC_UInt cMASK_ENDIANESS = (eC_UInt)(0x000000FF << cSHIFT_ENDIANESS); ///< mask for endianess in binary streaming header
const eC_UInt cMASK_ALIGNMENT = (eC_UInt)(0x000000FF << cSHIFT_ALIGNMENT); ///< mask for alignment in binary streaming header
const eC_UInt cMASK_VERSION = (eC_UInt)(0x0000FFFF << cSHIFT_VERSION);     ///< mask for version in binary streaming header

/** UUID */
struct UUID_t
{
    /** default constructor
    */
    UUID_t() :
        a(0),
        b(),
        c()
    {}

    /** less than operator
    @param rhs
    @return true if less than
    */
    inline bool operator<(const UUID_t& rhs) const
    {
        if (a < rhs.a)
            return true;
        else if (a > rhs.a)
            return false;
        else
        {
            for (eC_UInt i = 0; i < 2; ++i)
            {
                if (b[i] < rhs.b[i])
                    return true;
                if (b[i] > rhs.b[i])
                    return false;
            }

            for (eC_UInt i = 0; i < 6; ++i)
            {
                if (c[i] < rhs.c[i])
                    return true;
                if (c[i] > rhs.c[i])
                    return false;
            }
        }
        return false;
    }

    /** equal operator
    @param rhs
    @return true if equal
    */
    inline bool operator==(const UUID_t& rhs) const
    {
        if (a != rhs.a)
            return false;
        else
        {
            for (eC_UInt i = 0; i < 2; ++i)
            {
                if (b[i] != rhs.b[i])
                    return false;
            }

            for (eC_UInt i = 0; i < 6; ++i)
            {
                if (c[i] != rhs.c[i])
                    return false;
            }
        }
        return true;
    }

    eC_UInt a; ///< a part
    eC_UShort b[3]; ///< b part
    eC_UByte c[6]; ///< c part
};

#endif
