/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __WORKING_MEMORY__
#define __WORKING_MEMORY__

#include "eC_Types.h"
#include "eC_TArray.h"

// ---------------------------------------------------------------
// Working-Memory
// ---------------------------------------------------------------

/** this class provides a helper for working memories
*/
class WorkingMemory
{
public:
    /** Initialize the buffer for working memory
    @param uiSize initial size of buffer
    */
    WorkingMemory(const eC_UInt& uiSize);

    /** Get a portion of the working memory. This memory can be used for frequent actions
    like creating vertices or other temporary buffers for graphical operations.
    It is also possible to acquire multiple memory-blocks for an operation.

    If the memory-block will not fit into the buffer of working memory additional buffers
    are created.

    @param uiSize size of memory-block
    @return memory
    */
    void* Acquire(const eC_UInt& uiSize);

    /** Release all memory-blocks acquired from working memory
    This will reset the offset inside the working memory, so that
    the next acquisition will start from the beginning of the buffer.
    */
    void Release();

    /** return maximum buffer count
    @return max buffer count
    */
    eC_UInt GetMaxBufferCount() const
    {
        return m_uiMaxBufferCount;
    }

private:
    void AddNewBuffer(const eC_UInt& uiSize);

private:
    eC_UInt m_uiInitialSize; ///< initial size of buffer
    eC_TArray<eC_UByte*> m_pubBuffers; ///< buffers
    eC_TArray<eC_UInt> m_uiBufferSizes; ///< size of buffers
    eC_TArray<eC_UInt> m_uiCurrentOffset; ///< current offset in buffer

    eC_UInt m_uiMaxBufferCount; ///< maximum buffer count
};

#endif
