/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

/**
@warning Always include this file as the LAST included header in your .cpp files!
*/
#ifndef __GUI_MEM_LEAK_WATCHER_H__
#define __GUI_MEM_LEAK_WATCHER_H__

// include custom-memory-mapping
#include "eC_CustomMemoryMapping.h"

#include  <stddef.h>
#include "GUIConfigDebug.h"

#include "GUIMemLeakFunctions.h"

// if you even want to overload the c-functions (malloc, realloc and free)
// activate the define GUILIANI_OVERLOAD_C_FUNCS in GUIConfigDebug.h

// Disable warning "4291: no matching operator delete found; memory will..."
// on Visual C++ Compilers.
#if defined(eC_TARGET_ENV_WIN32)
#ifdef eC_TARGET_COMPILER_MSVC
    #pragma warning(disable : 4291)
#endif
#endif

#ifdef GUILIANI_OVERLOAD_CPP_FUNCS
// undef operators
#ifdef new
    #undef new
#endif

#ifdef delete
    #undef delete
#endif

// redefine operators
#define new new(__FILE__,__LINE__,__FUNCTION__)
#define delete Guiliani_info(__FILE__,__LINE__,__FUNCTION__),delete
#endif

#ifdef GUILIANI_OVERLOAD_C_FUNCS
// undef functions
#ifdef malloc
    #undef malloc
#endif

#ifdef free
    #undef free
#endif

#ifdef realloc
    #undef realloc
#endif

// redefine functions
#define malloc(x) Guiliani_malloc(x,__FILE__,__LINE__,__FUNCTION__)
#define free(x) Guiliani_free(x,__FILE__,__LINE__,__FUNCTION__)
#define realloc(x,y) Guiliani_realloc(x,y,__FILE__,__LINE__,__FUNCTION__)
#endif

#endif
