/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GUIBITMAPBLU_H__
#define __GUIBITMAPBLU_H__

#include "GUIBitmapPlane.h"
#include "blu_types.h"

/** @brief The BitmapPlane Class for BLU
<h3>Platform Specific Limitations </h3>
*/

// @guiliani_doxygen limitations CGfxWrapBLU
/**
 Transparencies are not supported for GUIBitmapPlane_BLU. (Class CGUIBitmapPlane_BLU)
 Roation is not supported
*/
// @endguiliani_doxygen
/**
    See @ref subsec_limitations_CGfxWrapBLU "CGUIBitmapPlane_BLU Platform Specific Limitations"
    @ingroup GUILIANI_BITMAPPLANES
*/
class CGUIBitmapPlane_BLU : public CGUIBitmapPlane
{
    friend class CGfxWrapBLU; ///< for private access

public:
    void DoDraw();

    /** Please note: In BLU only ONE alpha value for the whole plane is supported.
    This implementation takes the value in "ubTopLeft"
    @param ubTopLeft
    @param ubBottomLeft
    @param ubBottomRight
    @param ubTopRight
    */
    virtual void SetAlphaValues(
        const eC_UByte ubTopLeft, const eC_UByte ubBottomLeft,
        const eC_UByte ubBottomRight, const eC_UByte ubTopRight)
    {
        m_ubAlpha = ubTopLeft;
    }

    virtual void Clear(const eC_UByte ubRed, const eC_UByte ubGreen, const eC_UByte ubBlue, const eC_UByte ubAlpha);

    virtual void GetData(eC_TArray<eC_UByte>* pkData);

    /** Get data
    @return pixel-data in BLU-format
    */
    blu_bitmap_t* GetPixelData() { return &m_kBLUBitmap; }

protected:
    /** Constructor
        @param uiNewBitmapID Bitmap ID
        @param rkParentObject Parent object
        @param vWidth Width
        @param vHeight Height
    */
    CGUIBitmapPlane_BLU(GUIBitmapPlaneID_t uiNewBitmapID, const CGUIObject& rkParentObject, eC_Value vWidth, eC_Value vHeight);
    virtual ~CGUIBitmapPlane_BLU();

    /** Create texture for bitmap-plane
    */
    void CreateTexture();

private:
    eC_UByte m_ubAlpha; ///< Alpha value 0-255

    eC_UInt m_uiDataSize; ///< size of data
    eC_UByte* m_pkData; ///< pointer to data

    blu_bitmap_t m_kBLUBitmap; ///< blu-bitmap
};
#endif
