/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __DAVESCREEN_H__
#define __DAVESCREEN_H__

#include "GUIRect.h"

#include "eC_Types.h"

#include "dave_base.h"
#include "dave_driver.h"

/** Class to manage screens in D/AVE
*/
class CDAVEScreen
{
public:
    /** Constructor
    @param pkD1Handle
    @param pkD2Handle
    @param iD1Mode
    @param iD2Mode
    @param uiWidth
    @param uiHeight
    @param uiBytesPerPixel
    @param uiBufferCount
    @param ppBuffers
    */
    CDAVEScreen(
        d1_device* pkD1Handle,
        d2_device* pkD2Handle,
        const eC_Int& iD1Mode,
        const eC_Int& iD2Mode,
        const eC_UInt& uiWidth,
        const eC_UInt& uiHeight,
        const eC_UInt& uiBytesPerPixel,
        const eC_UInt& uiBufferCount,
        void** ppBuffers);
    /// Destructor
    virtual ~CDAVEScreen();

    /**
    Clear the screens
    */
    void Clear();

    /** Flip the screen
    */
    void Flip();

    /** Merge invalidated regions from display-buffer to render-buffer
    needs to be done AFTER flipping of the buffers.
    @param kInvalidatedRect area of invalidation
    @param bCopyContents true if screen should be copied
    */
    void Merge(
        const CGUIRect& kInvalidatedRect,
        const eC_Bool& bCopyContents = true);

    /** Return currently active render-buffer
    @return render-buffer
    */
    void* GetRenderBuffer();

    /** Return currently active display-buffer
    @return display-buffer
    */
    void* GetDisplayBuffer();

    /** Return number of buffers
    @return number of buffers
    */
    eC_UInt GetBufferCount() const;

    /** Return width of screen
    @return width
    */
    eC_UInt GetWidth() const;

    /** Return heihgt of screen
    @return height of screen
    */
    eC_UInt GetHeight() const;

    /** Return pitch of screen
    @return pitch of screen*/
    eC_UInt GetPitch() const;

    /** Return bits per pixel for screen
    @return bits per pixel
    */
    eC_UInt GetBitsPerPixel() const;

    /** Return D1-mode of screen
    @return D1-mode
    */
    eC_Int GetD1Mode() const;

    /** Return D2-mode of screen
    @return D2-mode
    */
    eC_Int GetD2Mode() const;

private:
    // no copy constructor
    CDAVEScreen(const CDAVEScreen&);

private:
    d1_device* m_pkD1Handle;
    d2_device* m_pkD2Handle;
    eC_Int m_iD1Mode;
    eC_Int m_iD2Mode;
    eC_UInt m_uiWidth;
    eC_UInt m_uiHeight;
    eC_UInt m_uiBytesPerPixel;
    eC_UInt m_uiBufferCount;
    void** m_ppBuffers;

    eC_UInt m_uiRenderBuffer;
    eC_UInt m_uiDisplayBuffer;
    bool m_bAllocated;
};

#endif
