/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GUI_IMAGEDATADAVE_H__
#define __GUI_IMAGEDATADAVE_H__

#include "GUIImageData.h"

/**
DAVE-specific implementation of CGUIImageData

this class will handle images with DAVE-specific additions
*/
class CGUIImageDataDAVE : public CGUIImageData
{
public:
    CGUIImageDataDAVE();

    virtual ~CGUIImageDataDAVE();

    /** Set if data is runlength-encoded
    @param bRLE
    */
    void SetRLE(const eC_Bool& bRLE);

    /** Return if data is RLE
    @return true if data is RLE
    */
    eC_Bool IsRLE() const;

    /** Set if CLUT is active or not
    @param bClut true if CLUT is present
    */
    void SetClut(const eC_Bool& bClut);

    /** Return if CLUT is present
    @return true if present
    */
    eC_Bool HasClut() const;

    /** Set data for color-lookup-table
    @param puiClutData address
    @param uiClutDataSize size
    */
    void SetClutData(eC_UInt* puiClutData, const eC_UInt& uiClutDataSize);

    /** Return the color-lookup-table
    @return address
    */
    void* GetClutData() const;

    /** Return the size of the color-lookup-table
    @return size
    */
    eC_UInt GetClutDataSize() const;

    /** Set the pixel-data of image
    @param pubPixelData address of data
    @param uiPixelDataSize size of data
    */
    void SetPixelData(eC_UByte* pubPixelData, const eC_UInt& uiPixelDataSize);

    /** Return the address for pixel-data
    @return address
    */
    void* GetPixelData() const;

    /** Return the size of pixel-data
    @return size of pixel-data
    */
    eC_UInt GetPixelDataSize() const;

private:
    eC_Bool m_bIsRLE;

    eC_Bool m_bUseClut; ///< color-lookup-table used
    eC_UInt* m_puiClutData; ///< pointer to clut
    eC_UInt m_uiClutDataSize; ///< size of clut
    eC_UInt m_uiPixelDataSize; ///< size of pixels
};

#endif
