/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __FNT_WRAP_GLYPHLIB_GL_H__
#define __FNT_WRAP_GLYPHLIB_GL_H__

#include "FntWrapGlyphLib.h"
#include "FntWrapGLFontAtlas.h"

/// Base class for GlyphLib-based font wrapper using GL textures.
/** This class uses the class FntWrapGlyphLibGLCache for displaying GlyphLib
    fonts using the OpenGL-Wrapper.
  */
class CFntWrapGlyphLibGL : public CFntWrapGlyphLib
{
public:
    /** Creates the instance of this wrapper.
        @param uiGlyphCacheInitialSize initial size for glyph-cache
        @param bGlyphCacheAutoResize if true the cache will be resized if necessary
        @param uiGlyphCacheResizeBy amount of glyph to be resized
        @return True if successful, False otherwise
    */
    static eC_Bool CreateInstance(
        const eC_UInt& uiGlyphCacheInitialSize = 32,
        const eC_Bool& bGlyphCacheAutoResize = true,
        const eC_UInt& uiGlyphCacheResizeBy = 10);

protected:
    virtual void LoadFont(const FontResource_t &eFontID);
    virtual void UnloadFont(const FontResource_t &eFontID);

    virtual void GetGlyphMetrics(const eC_Char cChar, GlyphMetrics_t& kMetrics);

    virtual eC_Int GetAscender() const;
    virtual eC_Int GetDescender() const;
    virtual eC_Int GetInternalLeading() const;

    virtual eC_Bool SetNOFFonts(const eC_UInt uiNOFFonts);

    /// The font cache array is an eC_TArray of CFntWrapGlyphLibGLCache pointers
    eC_TArray<CFntWrapGLFontAtlas*> m_apGLFontAtlas;

private:
    /** Constructor.
    @param uiGlyphCacheInitialSize initial size for glyph-cache
    @param bGlyphCacheAutoResize if true the cache will be resized if necessary
    @param uiGlyphCacheResizeBy amount of glyph to be resized
    */
    CFntWrapGlyphLibGL(
        const eC_UInt& uiGlyphCacheInitialSize,
        const eC_Bool& bGlyphCacheAutoResize,
        const eC_UInt& uiGlyphCacheResizeBy);

    /// Destructor.
    virtual ~CFntWrapGlyphLibGL(void);

    virtual void RenderTextInternal(
        const eC_String *lpString,
        const TextOperation_t &eOperation,
        const eC_Value& vX1,
        const eC_Value& vY1,
        eC_Value& vWidth,
        eC_Value& vHeight,
        eC_Value vWidthMax,
        eC_UInt &uiNumFittingChars);

    virtual void RenderGlyphToScreen(const eC_Value& vX1, const eC_Value& vY1, CGUIImageData& kGlyphImage) {}

    void AddGlyph(
        eC_TArray<eC_Value>& avVertices,
        const eC_UInt& uiArrayIndex,
        const eC_Value& vX,
        const eC_Value& vY,
        const eC_Int& iWidth,
        const eC_Int& iHeight,
        const eC_Value& vULeft,
        const eC_Value& vURight,
        const eC_Value& vVTop,
        const eC_Value& vVBottom);

    /** Copy-constructor.
        Dummy implementation to avoid unintended use of compiler-generated default
    */
    CFntWrapGlyphLibGL(const CFntWrapGlyphLibGL& kSource);

    /** Operator= method.
        Dummy implementation to avoid unintended use of compiler-generated default
    */
    CFntWrapGlyphLibGL& operator=(const CFntWrapGlyphLibGL& kSource);

private:
    eC_UInt m_uiGlyphCacheInitialSize;
    eC_Bool m_bGlyphCacheAutoResize;
    eC_UInt m_uiGlyphCacheResizeBy;
};
#endif
