/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __FNTWRAP_GLYPHLIB_GL_FONTATLAS__H__
#define __FNTWRAP_GLYPHLIB_GL_FONTATLAS__H__

#include "FntWrapGLFontAtlas.h"

#include "eC_Types.h"
#include "eC_TArray.h"
#include "GUIFontResource.h"

#include "glyphlib.h"
#include "gui_font.h"

/** This class is used by FntWrapFreeTypeGL. It stores the texture-atlas information for one Font.
*/
class CFntWrapGlyphLibGLFontAtlas : public CFntWrapGLFontAtlas
{
public:
    /** Constructor
    @param reFontID ID of selected font.
    @param uiNumberOfRows Initial number of rows for the texture atlas.
    @param pkFont Pointer of initialized FreeType library.
    */
    CFntWrapGlyphLibGLFontAtlas(
        const FontResource_t& reFontID,
        const eC_UInt& uiNumberOfRows,
        gui_font_t* pkFont);

    /** Destructor */
    virtual ~CFntWrapGlyphLibGLFontAtlas();

    /** Returns the handle of this font
    @return this font's handle.
    */
    virtual void* GetFace() { return (void*)m_pkFontHandle->font_handle; }

protected:
    /** Adds a character to the cache array and sorts array afterwards.
    @param uiUnicode Unicode to add to array.
    @return Struct of just added character.
    */
    virtual FontAtlasRecord_t AddGlyph(const eC_UInt &uiUnicode);

private:
    /** Initializes this class by opening the font face and setting parameters.
    */
    void Init();

    /** Copy-constructor.
    Dummy implementation to avoid unintended use of compiler-generated default    */
    CFntWrapGlyphLibGLFontAtlas(const CFntWrapGlyphLibGLFontAtlas& kSource);

    /** Operator= method.
    Dummy implementation to avoid unintended use of compiler-generated default    */
    CFntWrapGlyphLibGLFontAtlas& operator=(const CFntWrapGlyphLibGLFontAtlas& kSource);

private:
    gui_font_t* m_pkFontHandle;
};
#endif
