/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GL_API_MAPPINGS_H__
#define __GL_API_MAPPINGS_H__

#define GETGFX_GL static_cast<CGfxWrapGL&>(GETGFX)

#if defined (GUILIANI_USE_OGL)
    #include "OGL_API.h"
#elif defined(GUILIANI_USE_OGLES11) || defined(GUILIANI_USE_OGLES2)
    #include "OGLES_API.h"
#endif

// mapping for color-formats
#ifndef GL_RGB5
#define GL_RGB5 GL_RGB
#endif
#ifndef GL_RGB8
#define GL_RGB8 GL_RGB
#endif

// Macro to simplify the call of the Switch3D function.
#define SWITCH_TO_3D GETGFX_GL.Switch3D(GETGFX.GetVirtualScreenWidth(), \
    GETGFX.GetVirtualScreenHeight(), GetAbsXPos(), GetAbsYPos(), GetRelZPos(), \
    GetZeroZ(), GetWidth(), GetHeight(), GetInitWidth(), GetInitHeight(), \
    GetRotateX(), GetRotateY(), GetRotateZ(), GetNearPlane(), GetFarPlane(), \
    GetPerspectiveAngle())

#if defined SYSTEM_NAME_DARWIN
#define glBindBuffer_Guiliani glBindBuffer
#define glBufferData_Guiliani glBufferData
#define glGenBuffers_Guiliani glGenBuffers
#define glDeleteBuffers_Guiliani glDeleteBuffers
#define glGenFramebuffers_Guiliani glGenFramebuffers
#define glBindFramebuffer_Guiliani glBindFramebuffer
#define glFramebufferTexture2D_Guiliani glFramebufferTexture2D
#define glCheckFramebufferStatus_Guiliani glCheckFramebufferStatus
#define glDeleteFramebuffers_Guiliani glDeleteFramebuffers
#endif

// Type for the 3d matrix
const eC_UByte cubNOFMatrixElements = 4;
typedef eC_Value Matrix3D_t[cubNOFMatrixElements][cubNOFMatrixElements];

// the identity matrix
const Matrix3D_t cavIdentityMatrix =
{
    { eC_FromInt(1), eC_FromInt(0), eC_FromInt(0), eC_FromInt(0) },
    { eC_FromInt(0), eC_FromInt(1), eC_FromInt(0), eC_FromInt(0) },
    { eC_FromInt(0), eC_FromInt(0), eC_FromInt(1), eC_FromInt(0) },
    { eC_FromInt(0), eC_FromInt(0), eC_FromInt(0), eC_FromInt(1) }
};

#endif //#ifndef __GL_API_MAPPINGS_H__
