/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#if !defined __GFXWRAPOGLES1_1__H__
#define __GFXWRAPOGLES1_1__H__

#include "GfxWrapOGLES.h"

class CGUIBitmapPlane_PBuffer;

/// OpenGL ES 1.x graphics wrapper implementation with PBuffer support.

/** This graphic wrapper is derived from the CGfxWrapOGLES base class and uses PBuffer instead of the workaround. */
class CGfxWrapOGLES1_1 : public CGfxWrapOGLES
{
public:
    friend class CGUIBitmapPlane_PBuffer; ///< for private access

    /** Singleton for creating a GfxWrapOGLES object and change the pointer in
        base class to this graphic wrap.
        @param pkGfxEnv
        @return True if an instance was created by this call, otherwise False.
    */
    static eC_Bool CreateInstance(CGfxEnv* pkGfxEnv);

    virtual void ReadTextureData(
        const eC_UInt& uiTexID,
        const eC_UInt& uiXPos, const eC_UInt& uiYPos,
        const eC_UInt& uiWidth, const eC_UInt& uiHeight, void* pkData);

protected:
    /** Constructor
        @param pkGfxEnv
    */
    CGfxWrapOGLES1_1(CGfxEnv* pkGfxEnv);

    /** Destructor */
    virtual ~CGfxWrapOGLES1_1();

    /** Creates the graphic wrapper dependant bitmap (PBuffer in this case)
        @param[in] uiNewBitmapID Bitmap plane id
        @param[in] rkParentObject Parent object
        @param[in] vWidth Width of bitmap plane
        @param[in] vHeight Height of bitmap plane
        @return The created bitmap
    */
    virtual CGUIBitmapPlane* CreateGfxDependentBitmap(
        GUIBitmapPlaneID_t uiNewBitmapID,
        const CGUIObject& rkParentObject,
        const eC_Value &vWidth, const eC_Value &vHeight);
};

#endif //#if !defined __GFXWRAPOGLES1_1__H__
