/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __GFXWRAP_OGLES2_H__
#define __GFXWRAP_OGLES2_H__

#include "GfxWrapOGLES.h"

/// OpenGL ES 2.x graphics wrapper implementation with FBO support.

/** This graphics wrapper uses OpenGL ES 2.x
    BitmapPlanes are realized via Framebuffer-Objects.
*/
class CGfxWrapOGLES2 : public CGfxWrapOGLES
{
public:
    /** Singleton for creating a GfxWrapOGLES object and change the pointer in
        base class to this graphic wrap.
        @param pkGfxEnv environment to use
        @return True if an instance was created by this call, otherwise False.
    */
    static eC_Bool CreateInstance(CGfxEnv* pkGfxEnv);

    virtual void ReadTextureData(
        const eC_UInt& uiTexID,
        const eC_UInt& uiXPos, const eC_UInt& uiYPos,
        const eC_UInt& uiWidth, const eC_UInt& uiHeight, void* pkData);

protected:
    /** Constructor
        @param pkGfxEnv environment to use
    */
    CGfxWrapOGLES2(CGfxEnv* pkGfxEnv);

    /** Destructor */
    virtual ~CGfxWrapOGLES2();

    CGUIBitmapPlane* CreateGfxDependentBitmap(
        GUIBitmapPlaneID_t uiNewBitmapID,
        const CGUIObject& rkParentObject,
        const eC_Value &vWidth, const eC_Value &vHeight);
};

#endif //#if !defined __GFXWRAPOGLES2__H__
