/*
* Copyright (C) TES Electronic Solutions GmbH,
* All Rights Reserved.
* Contact: info@guiliani.de
*
* This file is part of the Guiliani HMI framework
* for the development of graphical user interfaces on embedded systems.
*/

#ifndef __FONTCACHE_FREETYPE_H__
#define __FONTCACHE_FREETYPE_H__

#include "FontCache.h"

//FreeType includes
#include "ft2build.h"
#include FT_GLYPH_H

#include "eC_Types.h"
#include "eC_TArray.h"
#include "GUIImageData.h"
#include "GUIFontResource.h"

/** This class is used to cache glyphs loaded by FreeType. It stores the cache information for
one Font.
*/
class CFontCacheFreeType : public CFontCache
{
public:
    /** Constructor
    @param pFTLib Pointer of initialized FreeType library.
    @param eFontID ID of selected font.
    @param uiInitialCacheSize initial number of simultaneously cached characters. If parameter
    equals zero the cache will automatically be resized to be
    large enough to contain all required characters.
    @param bAutoResize if true the cache gets resized if not big enough to hold new entries
    @param uiResizeBy number of elements which are added on resize
    */
    CFontCacheFreeType(
        const FT_Library& pFTLib,
        const FontResource_t& eFontID,
        const eC_UInt& uiInitialCacheSize,
        const eC_Bool& bAutoResize,
        const eC_UInt& uiResizeBy);

    /** Destructor */
    virtual ~CFontCacheFreeType();

protected:
    /** Adds a character to the cache array and sorts array afterwards.
    @param uiUnicode Unicode to add to array.
    @param uiPos Position in array where the character should be added.
    @return Struct of just added character.
    */
    virtual CachedChar_t AddChar(const eC_UInt &uiUnicode, const eC_UInt &uiPos);

    /** Copy-constructor.
    Dummy implementation to avoid unintended use of compiler-generated default
    @param kSource
    */
    CFontCacheFreeType(const CFontCacheFreeType& kSource);

    /** Operator= method.
    Dummy implementation to avoid unintended use of compiler-generated default
    @param kSource
    @return
    */
    CFontCacheFreeType& operator=(const CFontCacheFreeType& kSource);

private:
    /** initialize
    @param pFTLib
    */
    void Init(const FT_Library& pFTLib);
};

#endif
