/**
 * @file max9867.h
 * @brief MAX9867 audio CODEC
 *
 * @section License
 *
 * Copyright (C) 2010-2015 Oryx Embedded SARL. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * @author Oryx Embedded SARL (www.oryx-embedded.com)
 * @version 1.6.0
 **/

#ifndef _MAX9867_H
#define _MAX9867_H

// Dependencies
#include "m13_ra6m3_ek_status.h"



// MAX9867 registers
#define MAX9867_STATUS                            0x00
#define MAX9867_JACK_SENSE                        0x01
#define MAX9867_AUX_HIGH                          0x02
#define MAX9867_AUX_LOW                           0x03
#define MAX9867_INT_EN                            0x04
#define MAX9867_SYS_CLOCK                         0x05
#define MAX9867_STR_AUD_CLK_CTRL_H                0x06
#define MAX9867_STR_AUD_CLK_CTRL_L                0x07
#define MAX9867_DIG_AUD_IF1                       0x08
#define MAX9867_DIG_AUD_IF2                       0x09
#define MAX9867_CODEC_FL                          0x0A
#define MAX9867_SIDETONE                          0x0B
#define MAX9867_DAC_LVL                           0x0C
#define MAX9867_ADC_LVL                           0x0D
#define MAX9867_LINE_INL_LVL                      0x0E
#define MAX9867_LINE_INR_LVL                      0x0F
#define MAX9867_LVOL_CTRL                         0x10
#define MAX9867_RVOL_CTRL                         0x11
#define MAX9867_MICL_GAIN                         0x12
#define MAX9867_MICR_GAIN                         0x13
#define MAX9867_ADC_IN                            0x14
#define MAX9867_MIC                               0x15
#define MAX9867_MODE                              0x16
#define MAX9867_SYS_SHDN                          0x17
#define MAX9867_REVISION                          0xFF

// Status register
#define MAX9867_STATUS_CLD                        0x80
#define MAX9867_STATUS_SLD                        0x40
#define MAX9867_STATUS_ULK                        0x20
#define MAX9867_STATUS_JDET                       0x02

// Jack Sense register
#define MAX9867_JACK_SENSE_LSNS                   0x80
#define MAX9867_JACK_SENSE_JKSNS                  0x40
#define MAX9867_JACK_SENSE_JKMIC                  0x20

// AUX High register
#define MAX9867_AUX_HIGH_AUX_MSB                  0xFF

// AUX Low register
#define MAX9867_AUX_LOW_AUX_LSB                   0xFF

// Interrupt Enable register
#define MAX9867_INT_EN_ICLD                       0x80
#define MAX9867_INT_EN_ISLD                       0x40
#define MAX9867_INT_EN_IULK                       0x20
#define MAX9867_INT_EN_SDODLY                     0x04
#define MAX9867_INT_EN_IJDET                      0x02

// System Clock register
#define MAX9867_SYS_CLOCK_PSCLK                   0x30
#define MAX9867_SYS_CLOCK_PSCLK_DIS               0x00
#define MAX9867_SYS_CLOCK_PSCLK_10_20MHZ          0x10
#define MAX9867_SYS_CLOCK_PSCLK_20_40MHZ          0x20
#define MAX9867_SYS_CLOCK_PSCLK_40_60MHZ          0x30
#define MAX9867_SYS_CLOCK_FREQ                    0x0F
#define MAX9867_SYS_CLOCK_FREQ_PLL_MODE           0x00
#define MAX9867_SYS_CLOCK_FREQ_1500               0x08
#define MAX9867_SYS_CLOCK_FREQ_0750               0x09
#define MAX9867_SYS_CLOCK_FREQ_1625               0x0A
#define MAX9867_SYS_CLOCK_FREQ_0812               0x0B
#define MAX9867_SYS_CLOCK_FREQ_2000               0x0C
#define MAX9867_SYS_CLOCK_FREQ_1000               0x0D
#define MAX9867_SYS_CLOCK_FREQ_2400               0x0E
#define MAX9867_SYS_CLOCK_FREQ_1200               0x0F

// Stereo Audio Clock Control High & Low register
#define MAX9867_STR_AUD_CLK_CTRL_PLL              0x8000
#define MAX9867_STR_AUD_CLK_CTRL_NI               0x7FFF
#define MAX9867_STR_AUD_CLK_CTRL_NI_13MHZ_8KHZ    0x0F20
#define MAX9867_STR_AUD_CLK_CTRL_NI_13MHZ_16KHZ   0x1E3F
#define MAX9867_STR_AUD_CLK_CTRL_NI_13MHZ_24KHZ   0x2D5F
#define MAX9867_STR_AUD_CLK_CTRL_NI_13MHZ_32KHZ   0x3C7F
#define MAX9867_STR_AUD_CLK_CTRL_NI_13MHZ_44_1KHZ 0x535F
#define MAX9867_STR_AUD_CLK_CTRL_NI_13MHZ_48KHZ   0x5ABE

// Digital Audio Interface 1 register
#define MAX9867_DIG_AUD_IF1_MAS                   0x80
#define MAX9867_DIG_AUD_IF1_WCI                   0x40
#define MAX9867_DIG_AUD_IF1_BCI                   0x20
#define MAX9867_DIG_AUD_IF1_DLY                   0x10
#define MAX9867_DIG_AUD_IF1_HIZOFF                0x08
#define MAX9867_DIG_AUD_IF1_TDM                   0x04

// Digital Audio Interface 2 register
#define MAX9867_DIG_AUD_IF2_LVOLFIX               0x10
#define MAX9867_DIG_AUD_IF2_DMONO                 0x08
#define MAX9867_DIG_AUD_IF2_BSEL                  0x07
#define MAX9867_DIG_AUD_IF2_BSEL_OFF              0x00
#define MAX9867_DIG_AUD_IF2_BSEL_64X_LRCLK        0x01
#define MAX9867_DIG_AUD_IF2_BSEL_48X_LRCLK        0x02
#define MAX9867_DIG_AUD_IF2_BSEL_PCLK_DIV2        0x04
#define MAX9867_DIG_AUD_IF2_BSEL_PCLK_DIV4        0x05
#define MAX9867_DIG_AUD_IF2_BSEL_PCLK_DIV8        0x06
#define MAX9867_DIG_AUD_IF2_BSEL_PCLK_DIV16       0x07

// Digital Codec Filters register
#define MAX9867_CODEC_FL_MODE                     0x80
#define MAX9867_CODEC_FL_AVFLT                    0x70
#define MAX9867_CODEC_FL_DVFLT                    0x07
#define MAX9867_CODEC_FL_DVFLT_DISABLED           0x00
#define MAX9867_CODEC_FL_DVFLT_E_16_256_Y         0x01
#define MAX9867_CODEC_FL_DVFLT_B_16_500_N         0x02
#define MAX9867_CODEC_FL_DVFLT_E_8_256_Y          0x03
#define MAX9867_CODEC_FL_DVFLT_B_8_500_N          0x04
#define MAX9867_CODEC_FL_DVFLT_B_8_24_240_N       0x05

// Digital GAIN Sidetone register
#define MAX9867_SIDETONE_DSTS                     0xC0
#define MAX9867_SIDETONE_DSTS_NONE                0x00
#define MAX9867_SIDETONE_DSTS_ADCL                0x40
#define MAX9867_SIDETONE_DSTS_ADCR                0x80
#define MAX9867_SIDETONE_DSTS_ADCLR               0xC0
#define MAX9867_SIDETONE_DVST                     0x1F

// Digital GAIN DAC Level register
#define MAX9867_DAC_LVL_DACM                      0x40
#define MAX9867_DAC_LVL_DACG                      0x30
#define MAX9867_DAC_LVL_DACG_0DB                  0x00
#define MAX9867_DAC_LVL_DACG_6DB                  0x10
#define MAX9867_DAC_LVL_DACG_12DB                 0x20
#define MAX9867_DAC_LVL_DACG_18DB                 0x30
#define MAX9867_DAC_LVL_DACA                      0x0F

// Digital GAIN ADC Level register
#define MAX9867_ADC_LVL_AVL                       0xF0
#define MAX9867_ADC_LVL_AVR                       0x0F

// Left-Line Input Level register
#define MAX9867_LINE_INL_LVL_LILM                 0x40
#define MAX9867_LINE_INL_LVL_LIGL                 0x0F

// Right-Line Input Level register
#define MAX9867_LINE_INR_LVL_LIRM                 0x40
#define MAX9867_LINE_INR_LVL_LIGR                 0x0F

// Left Volume Control register
#define MAX9867_LVOL_CTRL_VOLLM                   0x40
#define MAX9867_LVOL_CTRL_VOLL                    0x3F

// Right Volume Control register
#define MAX9867_RVOL_CTRL_VOLRM                   0x40
#define MAX9867_RVOL_CTRL_VOLR                    0x3F

// Left Microphone Gain register
#define MAX9867_MICL_GAIN_PALEN                   0x60
#define MAX9867_MICL_GAIN_PALEN_DISABLED          0x00
#define MAX9867_MICL_GAIN_PALEN_0DB               0x20
#define MAX9867_MICL_GAIN_PALEN_20DB              0x40
#define MAX9867_MICL_GAIN_PALEN_30DB              0x60
#define MAX9867_MICL_GAIN_PGAML                   0x1F

// Right Microphone Gain register
#define MAX9867_MICR_GAIN_PAREN                   0x60
#define MAX9867_MICR_GAIN_PAREN_DISABLED          0x00
#define MAX9867_MICR_GAIN_PAREN_0DB               0x20
#define MAX9867_MICR_GAIN_PAREN_20DB              0x40
#define MAX9867_MICR_GAIN_PAREN_30DB              0x60
#define MAX9867_MICR_GAIN_PGAMR                   0x1F

// ADC Input register
#define MAX9867_ADC_IN_MXINL                      0xC0
#define MAX9867_ADC_IN_MXINL_NONE                 0x00
#define MAX9867_ADC_IN_MXINL_LR_AMIC              0x40
#define MAX9867_ADC_IN_MXINL_LR_LINE_IN           0x80
#define MAX9867_ADC_IN_MXINL_LR_AMIC_LINE_IN      0xC0
#define MAX9867_ADC_IN_MXINR                      0x30
#define MAX9867_ADC_IN_MXINR_NONE                 0x00
#define MAX9867_ADC_IN_MXINR_LR_AMIC              0x10
#define MAX9867_ADC_IN_MXINR_LR_LINE_IN           0x20
#define MAX9867_ADC_IN_MXINR_LR_AMIC_LINE_IN      0x30
#define MAX9867_ADC_IN_AUXCAP                     0x08
#define MAX9867_ADC_IN_AUXGAIN                    0x04
#define MAX9867_ADC_IN_AUXCAL                     0x02
#define MAX9867_ADC_IN_AUXEN                      0x01

// Digital Microphone Input register
#define MAX9867_MIC_MICCLK                        0xC0
#define MAX9867_MIC_MICCLK_PCLK_DIV_8             0x00
#define MAX9867_MIC_MICCLK_PCLK_DIV_6             0x40
#define MAX9867_MIC_DIGMICL                       0x20
#define MAX9867_MIC_DIGMICR                       0x10

// Mode register
#define MAX9867_MODE_DSLEW                        0x80
#define MAX9867_MODE_VSEN                         0x40
#define MAX9867_MODE_ZDEN                         0x20
#define MAX9867_MODE_JDETEN                       0x08
#define MAX9867_MODE_HPMODE                       0x07
#define MAX9867_MODE_HPMODE_STER_DIFF_CLI         0x00
#define MAX9867_MODE_HPMODE_MONO_DIFF_CLI         0x01
#define MAX9867_MODE_HPMODE_STER_CAPA_CLI         0x02
#define MAX9867_MODE_HPMODE_MONO_CAPA_CLI         0x03
#define MAX9867_MODE_HPMODE_STER_SNGL_CLI         0x04
#define MAX9867_MODE_HPMODE_MONO_SNGL_CLI         0x05
#define MAX9867_MODE_HPMODE_STER_SNGL_FST         0x06
#define MAX9867_MODE_HPMODE_MONO_SNGL_FST         0x07

// System Shutdown register
#define MAX9867_SYS_SHDN_SHDN                     0x80
#define MAX9867_SYS_SHDN_LNLEN                    0x40
#define MAX9867_SYS_SHDN_LNREN                    0x20
#define MAX9867_SYS_SHDN_DALEN                    0x08
#define MAX9867_SYS_SHDN_DAREN                    0x04
#define MAX9867_SYS_SHDN_ADLEN                    0x02
#define MAX9867_SYS_SHDN_ADREN                    0x01

// Revision register
#define MAX9867_REVISION_REV                      0xFF
#define MAX9867_REVISION_REV_DEFAULT              0x42


// Audio related functions
bsp_status_t bsp_audio_init(void);
bsp_status_t bsp_audio_set_volume(uint8_t left, uint8_t right);
bsp_status_t bsp_audio_read_reg(uint8_t address, uint8_t *data);
bsp_status_t bsp_audio_write_reg(uint8_t address, uint8_t data);
// error_t bsp_audio_read_reg16(uint8_t address, uint16_t data);
bsp_status_t bsp_audio_dump_reg(void);
bsp_status_t bsp_audio_mute(void);

#endif
