/**
 * @file m13_ra6m3_ek_eeprom.h
 * @brief M13-RA6M3-EK eeprom BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#ifndef M13_RA6M3_EK_EEPROM_H
#define M13_RA6M3_EK_EEPROM_H

// Depencencies
#include "m13_ra6m3_ek_status.h"
#include <stdint.h>

// Macro definitions
#define EEPROM_MAX_SIZE      	 16384
#define EEPROM_PAGE_WRITE_BUF    16


// EEPROM BSP functions
bsp_status_t bsp_eeprom_init(void);
bsp_status_t bsp_eeprom_deinit(void);

bsp_status_t bsp_eeprom_write(uint16_t addr, const void *data, uint32_t size);
bsp_status_t bsp_eeprom_read(uint16_t addr, void *data, uint32_t size);

bsp_status_t bsp_eeprom_dump(void);
bsp_status_t bsp_eeprom_test(void);

void bsp_eeprom_write_delay(void);

#endif
