/**
 * @file m13_ra6m3_ek_touch.h
 * @brief M13-RA6M3-EK LCD Touch FT5206 description
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#ifndef _M13_RA6M3_EK_TOUCH_H
#define _M13_RA6M3_EK_TOUCH_H

//Dependencies
#include "m13_ra6m3_ek_status.h"
#include "m13_ra6m3_ek_pinout.h"
#include "m13_ra6m3_ek_i2c.h"


#define FT5206_TOUCH_XH(n) (FT5206_TOUCH1_XH + (((n) - 1) * 6))
#define FT5206_TOUCH_XL(n) (FT5206_TOUCH1_XL + (((n) - 1) * 6))

#define FT5206_TOUCH_YH(n) (FT5206_TOUCH1_YH + (((n) - 1) * 6))
#define FT5206_TOUCH_YL(n) (FT5206_TOUCH1_YL + (((n) - 1) * 6))

#define ID_FINGER(n) (_1ST_FINGER + (n))

// typedef definitions
typedef enum
{
    _NO_FINGER = 0,
    _1ST_FINGER,
    _2ND_FINGER,
    _3RD_FINGER,
    _4TH_FINGER,
    _5TH_FINGER,
} finger_id_t;


//FT5206 registers
#define FT5206_DEVICE_MODE                0x00
#define FT5206_GEST_ID                    0x01
#define FT5206_TD_STATUS                  0x02
#define FT5206_TOUCH1_XH                  0x03
#define FT5206_TOUCH1_XL                  0x04
#define FT5206_TOUCH1_YH                  0x05
#define FT5206_TOUCH1_YL                  0x06
#define FT5206_TOUCH2_XH                  0x09
#define FT5206_TOUCH2_XL                  0x0A
#define FT5206_TOUCH2_YH                  0x0B
#define FT5206_TOUCH2_YL                  0x0C
#define FT5206_TOUCH3_XH                  0x0F
#define FT5206_TOUCH3_XL                  0x10
#define FT5206_TOUCH3_YH                  0x11
#define FT5206_TOUCH3_YL                  0x12
#define FT5206_TOUCH4_XH                  0x15
#define FT5206_TOUCH4_XL                  0x16
#define FT5206_TOUCH4_YH                  0x17
#define FT5206_TOUCH4_YL                  0x18
#define FT5206_TOUCH5_XH                  0x1B
#define FT5206_TOUCH5_XL                  0x1C
#define FT5206_TOUCH5_YH                  0x1D
#define FT5206_TOUCH5_YL                  0x1E
#define FT5206_ID_G_THGROUP               0x80
#define FT5206_ID_G_THPEAK                0x81
#define FT5206_ID_G_THCAL                 0x82
#define FT5206_ID_G_THWATER               0x83
#define FT5206_ID_G_THTEMP                0x84
#define FT5206_ID_G_THDIFF                0x85
#define FT5206_ID_G_CTRL                  0x86
#define FT5206_ID_G_TIME_ENTER_MONITOR    0x87
#define FT5206_ID_G_PERIODACTIVE          0x88
#define FT5206_ID_G_PERIOD_MONITOR        0x89
#define FT5206_ID_G_AUTO_CLB_MODE         0xA0
#define FT5206_ID_G_LIB_VERSION_H         0xA1
#define FT5206_ID_G_LIB_VERSION_L         0xA2
#define FT5206_ID_G_CIPHER                0xA3
#define FT5206_ID_G_MODE                  0xA4
#define FT5206_ID_G_PMODE                 0xA5
#define FT5206_ID_G_FIRMID                0xA6
#define FT5206_ID_G_STATE                 0xA7
#define FT5206_ID_G_FT5201ID              0xA8
#define FT5206_ID_G_ERR                   0xA9
#define FT5206_ID_G_CLB                   0xAA

//Device mode register
#define FT5206_DEVICE_MODE_MODE           0x70
#define FT5206_DEVICE_MODE_MODE_NORMAL    0x00
#define FT5206_DEVICE_MODE_MODE_SYS_INFO  0x10
#define FT5206_DEVICE_MODE_MODE_TEST      0x40

//Gesture of a valid touch register
#define FT5206_GEST_ID_MOVE_UP            0x10
#define FT5206_GEST_ID_MOVE_LEFT          0x14
#define FT5206_GEST_ID_MOVE_DOWN          0x18
#define FT5206_GEST_ID_MOVE_RIGHT         0x1C
#define FT5206_GEST_ID_ZOOM_IN            0x48
#define FT5206_GEST_ID_ZOOM_OUT           0x49
#define FT5206_GEST_ID_NONE               0x00

// Touch Data status register To Confirm !
#define FT5206_TD_STATUS_FINGER1          0x01
#define FT5206_TD_STATUS_FINGER2          0x02
#define FT5206_TD_STATUS_FINGER3          0x03
#define FT5206_TD_STATUS_FINGER4          0x04
#define FT5206_TD_STATUS_FINGER5          0x05
#define FT5206_TD_STATUS_NONE             0x00

//Touch Data status register register
#define FT5206_TD_STATUS_NUM_OF_TOUCH     0x0F

//MSB Touch register mask
#define FT5206_TOUCH_XH_MSB_X_POS         0x0F
#define FT5206_TOUCH_YH_MSB_Y_POS         0x0F

//MSB of the X coordinate of the 1st touch point and the corresponding event flag register
#define FT5206_TOUCH1_XH_EVENT            0xC0
#define FT5206_TOUCH1_XH_EVENT_PUT_DOWN   0x00
#define FT5206_TOUCH1_XH_EVENT_PUT_UP     0x40
#define FT5206_TOUCH1_XH_EVENT_CONTACT    0x80
#define FT5206_TOUCH1_XH_EVENT_RESERVED   0xC0
#define FT5206_TOUCH1_XH_MSB_X_POS        0x0F

//MSB of the Y coordinate of the 1st touch point and corresponding touch ID register
#define FT5206_TOUCH1_YH_ID               0xF0
#define FT5206_TOUCH1_YH_MSB_Y_POS        0x0F

//MSB of the X coordinate of the 2nd touch point and the corresponding event flag register
#define FT5206_TOUCH2_XH_EVENT            0xC0
#define FT5206_TOUCH2_XH_EVENT_PUT_DOWN   0x00
#define FT5206_TOUCH2_XH_EVENT_PUT_UP     0x40
#define FT5206_TOUCH2_XH_EVENT_CONTACT    0x80
#define FT5206_TOUCH2_XH_EVENT_RESERVED   0xC0
#define FT5206_TOUCH2_XH_MSB_X_POS        0x0F

//MSB of the Y coordinate of the 2nd touch point and corresponding touch ID register
#define FT5206_TOUCH2_YH_ID               0xF0
#define FT5206_TOUCH2_YH_MSB_Y_POS        0x0F

//MSB of the X coordinate of the 3rd touch point and the corresponding event flag register
#define FT5206_TOUCH3_XH_EVENT            0xC0
#define FT5206_TOUCH3_XH_EVENT_PUT_DOWN   0x00
#define FT5206_TOUCH3_XH_EVENT_PUT_UP     0x40
#define FT5206_TOUCH3_XH_EVENT_CONTACT    0x80
#define FT5206_TOUCH3_XH_EVENT_RESERVED   0xC0
#define FT5206_TOUCH3_XH_MSB_X_POS        0x0F

//MSB of the Y coordinate of the 3rd touch point and corresponding touch ID register
#define FT5206_TOUCH3_YH_ID               0xF0
#define FT5206_TOUCH3_YH_MSB_Y_POS        0x0F

//MSB of the X coordinate of the 4th touch point and the corresponding event flag register
#define FT5206_TOUCH4_XH_EVENT            0xC0
#define FT5206_TOUCH4_XH_EVENT_PUT_DOWN   0x00
#define FT5206_TOUCH4_XH_EVENT_PUT_UP     0x40
#define FT5206_TOUCH4_XH_EVENT_CONTACT    0x80
#define FT5206_TOUCH4_XH_EVENT_RESERVED   0xC0
#define FT5206_TOUCH4_XH_MSB_X_POS        0x0F

//MSB of the Y coordinate of the 4th touch point and corresponding touch ID register
#define FT5206_TOUCH4_YH_ID               0xF0
#define FT5206_TOUCH4_YH_MSB_Y_POS        0x0F

//CMSB of the X coordinate of the 5th touch point and the corresponding event flag register
#define FT5206_TOUCH5_XH_EVENT            0xC0
#define FT5206_TOUCH5_XH_EVENT_PUT_DOWN   0x00
#define FT5206_TOUCH5_XH_EVENT_PUT_UP     0x40
#define FT5206_TOUCH5_XH_EVENT_CONTACT    0x80
#define FT5206_TOUCH5_XH_EVENT_RESERVED   0xC0
#define FT5206_TOUCH5_XH_MSB_X_POS        0x0F

//MSB of the Y coordinate of the 5th touch point and corresponding touch ID register
#define FT5206_TOUCH5_YH_ID               0xF0
#define FT5206_TOUCH5_YH_MSB_Y_POS        0x0F

//Auto calibration mode register
#define FT5206_ID_G_AUTO_CLB_MODE_ENABLE  0x00
#define FT5206_ID_G_AUTO_CLB_MODE_DISABLE 0xFF

//Vendor ID register
#define FT5206_ID_G_CIPHER_DEFAULT        0x55

//The interrupt status to host register
#define FT5206_ID_G_MODE_POLLING          0x00
#define FT5206_ID_G_MODE_TRIGGER          0x01

//Power Consume Mode register
#define FT5206_ID_G_PMODE_ACTIVE          0x00
#define FT5206_ID_G_PMODE_MONITOR         0x01
#define FT5206_ID_G_PMODE_HIBERNATE       0x05

//Running State register
#define FT5206_ID_G_STATE_CONFIGURE       0x00
#define FT5206_ID_G_STATE_WORK            0x01
#define FT5206_ID_G_STATE_CALIBRATION     0x02
#define FT5206_ID_G_STATE_FACTORY         0x03
#define FT5206_ID_G_STATE_AUTO_CALIB      0x04

//Error Code  register
#define FT5206_ID_G_ERR_OK                0x00
#define FT5206_ID_G_ERR_CHIP_REG_INCOS    0x03
#define FT5206_ID_G_ERR_CHIP_START_FAIL   0x05
#define FT5206_ID_G_ERR_NO_MATCH          0x1A

//----------------------------------
typedef enum
{
    TPEVT_ENTRY_NONE    = 0x0000,      /*!< None */
    TPEVT_ENTRY_UP      = 0x0001,      /*!< Up */
    TPEVT_ENTRY_DOWN    = 0x0002,      /*!< Down */
    TPEVT_ENTRY_MOVE    = 0x0004,      /*!< Move */
    TPEVT_ENTRY_ALL     = 0x0007,      /*!< All */
    TPEVT_ENTRY_UNKNOWN = 0x8000       /*!< internal event state */
} TpEvt_EntryType ;

typedef struct
{
    TpEvt_EntryType     eState;
    uint8_t				id;
    uint16_t            unPosX;
    uint16_t            unPosY;
    int 				first_touch;
} TP_TouchFinger_st;

//----------------------------------

typedef enum
{
	DOWN = 0x00,
	UP = 0x01,
	CONTACT = 0x02,
	RESERVED = 0x03
} EventType;

typedef struct
{
	uint8_t event;
	uint8_t id;
	uint16_t x;
	uint16_t y;
} touch_info_st;

//Touch related functions
bsp_status_t bsp_touch_init(void);
bsp_status_t bsp_touch_get_status(uint8_t *touch_status);
bsp_status_t bsp_touch_get_gest_id(uint8_t *gest_id);
bsp_status_t bsp_touch_get_data(uint8_t *touch_data, size_t size);
bsp_status_t bsp_touch_get_position(uint8_t *touch_data, uint8_t finger_id, uint16_t *x, uint16_t *y);
bsp_status_t bsp_touch_get_finger_event(uint8_t *touch_data, uint8_t finger_id, uint8_t *event);
bsp_status_t bsp_touch_get_finger_id(uint8_t *touch_data, uint8_t finger_id, uint8_t *id);
bsp_status_t bsp_touch_write_reg(uint8_t address, uint8_t data);
bsp_status_t bsp_touch_read_reg(uint8_t address, uint8_t *data);
bsp_status_t bsp_touch_read_regs(uint8_t address, uint8_t *data, size_t data_size);


#endif
