#include "m13_ra6m3_ek_touch.h"
#include "m13_ra6m3_ek_lcd.h"
#include "m13_ra6m3_ek_vcom.h"
#include "common_data.h"
#include <stdlib.h>
#include "touch_gslX680.h"

/* Touch Thread entry function */

extern uint16_t screen_rotation;
extern SemaphoreHandle_t g_touch_semaphore;
extern int32_t touch_read_request;

extern SemaphoreHandle_t i2c_channel_0_semaphore;

extern void touchevent_callback(TP_TouchFinger_st* pkTouchData);

touchevent_data_t touch_event;

/* pvParameters contains TaskHandle_t */
void touch_task(void *pvParameters)
{
    FSP_PARAMETER_NOT_USED (pvParameters);

    uint8_t touch_status = 0;
    uint8_t last_touch_status = 0;
    uint8_t touch_data[31];

    uint16_t x_pos[5];
    uint16_t y_pos[5];
    bsp_status_t error;

    touch_info_st touchinfo[5];

    //----------------------------------
    TP_TouchFinger_st  sTouchEvt;
    memset(&sTouchEvt, 0x00, sizeof(sTouchEvt));
    //----------------------------------

    // clear touch data
    memset(touch_data, 0xff, sizeof(touch_data));

    int first_touch = 1;

    while (1)
    {
        /* Wait for IRQ from touch controller. */
        xSemaphoreTake(g_touch_semaphore, portMAX_DELAY);

        xSemaphoreTake(i2c_channel_0_semaphore, portMAX_DELAY);

		bsp_touch_get_status(&touch_status);

        // Release semaphore
        xSemaphoreGive(i2c_channel_0_semaphore);

		if(touch_status >= 1 && touch_status <= 5)
		{
	        xSemaphoreTake(i2c_channel_0_semaphore, portMAX_DELAY);
			bsp_touch_get_data(touch_data, sizeof(touch_data));
	        xSemaphoreGive(i2c_channel_0_semaphore);
		}

    	uint8_t event, id;
        uint16_t xpos, ypos;

        //----------------------------------
        for (int i = 0; i < 5; ++i)
        {
            bsp_touch_get_finger_event(touch_data, ID_FINGER(i), &event);
            bsp_touch_get_finger_id(touch_data, ID_FINGER(i), &id);
            bsp_touch_get_position(touch_data, ID_FINGER(i), &xpos, &ypos);

            // new data
            if (
				(touchinfo[i].event != event) ||
				(touchinfo[i].id != id) ||
				(touchinfo[i].x != xpos) ||
				(touchinfo[i].y != ypos)
				)
            {
            	//PRINTF_INFO("i: %i e: %i id: %i x: %i y: %i\r\n", i, event, id, xpos, ypos);

        		sTouchEvt.id = id;
        		sTouchEvt.unPosX = xpos;
        		sTouchEvt.unPosY = ypos;
        		sTouchEvt.first_touch = first_touch;

            	if (event == DOWN)
            	{
            		// down
            		sTouchEvt.eState = TPEVT_ENTRY_DOWN;
            		touchevent_callback(&sTouchEvt);
            	}
            	else if (event == CONTACT)
            	{
            		// move
            		sTouchEvt.eState = TPEVT_ENTRY_MOVE;
            		touchevent_callback(&sTouchEvt);
            	}
            	else if (event == UP)
            	{
            		// up
            		sTouchEvt.eState = TPEVT_ENTRY_UP;
            		touchevent_callback(&sTouchEvt);
            	}
            	else
            	{
            		PRINTF_INFO("unknown event\r\n");
            	}

            	touchinfo[i].event = event;
            	touchinfo[i].id = id;
            	touchinfo[i].x = xpos;
            	touchinfo[i].y = ypos;

            	if (first_touch)
            		first_touch = 0;
            }
        }

        if (last_touch_status != touch_status)
        {
        	if (touch_status == 0)
        	{
        		// last up-event
                for (int i = 0; i < 5; ++i)
                {
                    bsp_touch_get_finger_event(touch_data, ID_FINGER(i), &event);
                    bsp_touch_get_finger_id(touch_data, ID_FINGER(i), &id);
                    bsp_touch_get_position(touch_data, ID_FINGER(i), &xpos, &ypos);

                    if (event == CONTACT)
                    {
                		// up
                		sTouchEvt.eState = TPEVT_ENTRY_UP;
                		sTouchEvt.id = id;
                		sTouchEvt.unPosX = xpos;
                		sTouchEvt.unPosY = ypos;

                		touchevent_callback(&sTouchEvt);

                    	touchinfo[i].event = event;
                    	touchinfo[i].id = id;
                    	touchinfo[i].x = xpos;
                    	touchinfo[i].y = ypos;

                    	first_touch = 1;
                    }
                }
        	}

        	last_touch_status = touch_status;
        }

        vTaskDelay(2 / portTICK_PERIOD_MS);
        //----------------------------------
    }
}
