/***********************************************************************************************************************
 * File Name    : emWin_thread_entry.c
 * Description  : Contains code to initialize emWin.
 **********************************************************************************************************************/
/***********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
 * SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
 ***********************************************************************************************************************/
#include "main_thread.h"
#include "common_data.h"
#include "r_display_api.h"
#include "r_glcdc.h"

extern "C"
{
#include "m13_ra6m3_ek_config.h"
#include "m13_ra6m3_ek_led.h"
#include "m13_ra6m3_ek_vcom.h"
#include "m13_ra6m3_ek_adc.h"
#include "m13_ra6m3_ek_acc.h"
#include "m13_ra6m3_ek_ethernet.h"
#include "m13_ra6m3_ek_touch.h"
#include "m13_ra6m3_ek_lcd.h"
#include "m13_ra6m3_ek_sdram.h"
#include "m13_ra6m3_ek_qspi.h"
#include "touch_gslX680.h"

// External task definitions
extern void ethernet_task(void *parameters);
extern void accelerometer_task(void *parameters);
extern void adc_task(void *parameters);
extern void lcd_task(void *parameters);
extern void touch_task(void *pvParameters);
extern void audio_task(void *pvParameters);

// framebuffer for D/AVE is placed in SDRAM
extern uint8_t lcd_fb_background [BSP_LCD_MAX_FRAME_BUFFER] [DISPLAY_BUFFER_STRIDE_BYTES_INPUT0 * 272];
// LCD-buffer is placed in SRAM
uint8_t screenbuffer[DISPLAY_BUFFER_STRIDE_BYTES_INPUT0 * 272] BSP_ALIGN_VARIABLE(64) BSP_PLACE_IN_SECTION(".bss");

extern glcdc_instance_ctrl_t g_display_ctrl;
extern const display_cfg_t g_display_cfg;

// accelerometer
extern int16_t ax;
extern int16_t ay;
extern int16_t az;
}

#include "stdio.h"

// Global variables
SemaphoreHandle_t i2c_channel_0_semaphore;

#define LCD_NUM_FRAMEBUFFERS 2
#define LCD_BYTES_PER_PIXEL (DISPLAY_BITS_PER_PIXEL_INPUT0 >> 3)

uint8_t BytesPerPixel = LCD_BYTES_PER_PIXEL;
uint16_t m_uiWidth = LCD_WIDTH;
uint16_t m_uiHeight = LCD_HEIGHT;

volatile uint8_t *RZAFrameBuffers[] =
{
     lcd_fb_background[0],
     lcd_fb_background[0]
};

int fb_index = 0;
unsigned char color1 = 0x88;
unsigned char color2 = 0x2f;

int global_x = 0;
int global_y = 0;

int currentcolor = 0xff88;

extern "C"
{
    volatile int stack_overflow_count = 0;
    void vApplicationStackOverflowHook( TaskHandle_t xTask, char * pcTaskName )
    {
        stack_overflow_count++;
    }

    void DrawDot(int x, int y, int color);
    void PutPixel(int x, int y, int color);
    static void _GraphicsHWInit (void);
    void CopyBuffer(int iX1, int iY1, int iRectWidth, int iRectHeight);

    void touchevent_callback(touchevent_data_t* pkTouchData)
    {
        int x = pkTouchData->x;
        int y = pkTouchData->y;

        if (pkTouchData->event == TOUCH_EVENT_UP)
        {
            global_x = 0;
            global_y = 0;
        }
        else
        {
            global_x = x;
            global_y = y;

            DrawDot(global_x, global_y, currentcolor);
            CopyBuffer(0, 0, 480, 272);

            currentcolor = currentcolor ^ 0x62fc;
        }
    }

    void PutPixel(int x, int y, int color)
    {
        if (
            (x >= 0 && x < LCD_WIDTH) &&
            (y >= 0 && y < LCD_HEIGHT)
            )
        {
            RZAFrameBuffers[fb_index][(y * LCD_WIDTH + x) * 2 + 0] = (color & 0xff00) >> 8;
            RZAFrameBuffers[fb_index][(y * LCD_WIDTH + x) * 2 + 1] = color & 0x0ff;
        }
    }

    void DrawDot(int x, int y, int color)
    {
        for (int currenty = y - 5; currenty <= y + 5; ++currenty)
        {
            for (int currentx = x - 5; currentx <= x + 5; ++currentx)
            {
                PutPixel(currentx, currenty, color);
            }
        }
    }

    /*********************************************************************
     *
     *       _GraphicsHWInit
     */
    static void _GraphicsHWInit (void)
    {
        bsp_lcd_io_init();

        memset((void*)screenbuffer, 0x3e, LCD_WIDTH * LCD_HEIGHT * LCD_BYTES_PER_PIXEL);
        for (int index = 0; index < LCD_NUM_FRAMEBUFFERS; ++index)
        {
            memset((void*)RZAFrameBuffers[index], 0x88, LCD_WIDTH * LCD_HEIGHT * LCD_BYTES_PER_PIXEL);
        }

        bsp_lcd_init();
        bsp_lcd_display_on();

        bsp_lcd_set_backlight((bsp_io_level_t)1);

        bsp_lcd_enable_double_buffer();

        bsp_lcd_set_display_buffer(fb_index);

        R_GLCDC_BufferChange(
            &g_display_ctrl,
            (uint8_t*)screenbuffer,
            DISPLAY_FRAME_LAYER_1
        );
    }

    void FlipFramebuffer()
    {
        vsync_wait();

        R_GLCDC_BufferChange(
            &g_display_ctrl,
            (uint8_t*)screenbuffer,
            DISPLAY_FRAME_LAYER_1
        );
    }

    void CopyBuffer(int iX1, int iY1, int iRectWidth, int iRectHeight)
    {
        uint8_t* pubSource = lcd_fb_background[fb_index] + (iY1 * m_uiWidth + iX1) * BytesPerPixel;
        uint8_t* pubDest = screenbuffer + (iY1 * m_uiWidth + iX1) * BytesPerPixel;
        for (uint16_t iLineIndex = 0; iLineIndex < iRectHeight; ++iLineIndex)
        {
            memcpy(pubDest, pubSource, iRectWidth * BytesPerPixel);

            pubSource += m_uiWidth * BytesPerPixel;
            pubDest += m_uiWidth * BytesPerPixel;
        }

        FlipFramebuffer();
    }
}

/* pvParameters contains TaskHandle_t */
void main_thread_entry(void *pvParameters)
{
    FSP_PARAMETER_NOT_USED (pvParameters);

    portBASE_TYPE status;
    TaskHandle_t task;

    //Initialize UART for debug
    bsp_vcom_init();

    // Open SCI channel 4
    bsp_vcom_open(BSP_VCOM_CHANNEL_4);

    // Configure the led pins
    bsp_led_io_init();

    // Configure the ADC pins
    bsp_adc_io_init();

    // Create Mutex for I2C channel 0 access
    i2c_channel_0_semaphore = xSemaphoreCreateMutex();

    // Configure i2c0 pins
    bsp_i2c_init(BSP_I2C_CHANNEL_0);

    // Initialize & configure the accelerometer
    bsp_acc_init();

    // Init touch
    bsp_touch_init();

#if (BSP_CONFIG_ENABLE_LCD_BUFFER_IN_SDRAM == 1 && BSP_CONFIG_ENABLE_HEAP_IN_SDRAM == 0 )
    bsp_sdram_init();
#endif

    // Init QSPI
    bsp_qspi_init();

    // Create Accelerometer task
    status = xTaskCreate(accelerometer_task, "Accelerometer Task", 1000, NULL, 1, &task);
    if( status != pdPASS )
    {
        PRINTF_INFO("Could not create Accelerometer Task \r\n");
    }

    // Create ADC task
    status = xTaskCreate(adc_task, "ADC Task", 1000, NULL, 1, &task);
    if( status != pdPASS )
    {
        PRINTF_INFO("Could not create ADC Task \r\n");
    }

    // Create Touch task
    status = xTaskCreate(touch_task, "Touch Task", 1000, NULL, 1, &task);
    if( status != pdPASS )
    {
        PRINTF_INFO("Could not create Touch Task \r\n");
    }

    _GraphicsHWInit();

    // Use main_thread_entry to blink the board leds
    while (1)
    {
        for (int y = 0; y < LCD_HEIGHT; ++y)
        {
            for (int x = 0; x < LCD_WIDTH / 2; ++x)
            {
                RZAFrameBuffers[fb_index][(y * LCD_WIDTH + x) * LCD_BYTES_PER_PIXEL + 0] = color1;
                RZAFrameBuffers[fb_index][(y * LCD_WIDTH + x) * LCD_BYTES_PER_PIXEL + 1] = color2;
            }
        }

        //_SwitchBuffersOnVSYNC(fb_index);

        color1 = (ax >> 8);
        color2 = (ay >> 8);

        //Turn on LED1 for 50ms)
        bsp_led_on(BSP_LED1);
        bsp_led_on(BSP_LED2);
        bsp_led_on(BSP_LED3);
        vTaskDelay(50);

        //Turn on LED2 for 950ms)
        bsp_led_off(BSP_LED1);
        bsp_led_off(BSP_LED2);
        bsp_led_off(BSP_LED3);
        vTaskDelay(950);
    }

}
