#ifndef __GUIINPUT_RA6M3_H__
#define __GUIINPUT_RA6M3_H__

extern "C"
{
    #include "touch_gslX680.h"
    #include "m13_ra6m3_ek_touch.h"
}

#include "eC_Semaphore.h"
#include "eC_Types.h"
#include "eC_TList_doubleLinked.h"
#include "GUIInputMedia.h"
#include "GUIEvent.h"

/** this class is used for the input-device on the RA6M3
*/
class CGUIInputRA6M3 : public CGUIInputMedia
{
public:
    /**
        Singleton for creating an input object and change the pointer in
        base class to input window.
        @param iTouchScreenX
        @param iTouchScreenY
        @param iTouchScreenWidth
        @param iTouchScreenHeight
        @param iTouchFilterDelta
     */
    static void CreateInstance(
        const eC_Int& iTouchScreenX,
        const eC_Int& iTouchScreenY,
        const eC_Int& iTouchScreenWidth,
        const eC_Int& iTouchScreenHeight,
        const eC_Int& iTouchFilterDelta);
    
    /** add a new event
    @param kTouchEvent
    */
    void AddEvent(TP_TouchFinger_st kTouchEvent);

protected:
    /** constructor
        @param iTouchScreenX
        @param iTouchScreenY
        @param iTouchScreenWidth
        @param iTouchScreenHeight
        @param iTouchFilterDelta
    */
    CGUIInputRA6M3(
        const eC_Int& iTouchScreenX,
        const eC_Int& iTouchScreenY,
        const eC_Int& iTouchScreenWidth,
        const eC_Int& iTouchScreenHeight,
        const eC_Int& iTouchFilterDelta);

    /** destructor
    */
    virtual ~CGUIInputRA6M3();

private:
    /**
        Copy-constructor.
        @param kSource Source object to be copied.
     */
    CGUIInputRA6M3(const CGUIInputRA6M3& kSource);

    /**
        Operator= method.
        @param kSource Source object to be copied.
        @return This instance.
     */
    CGUIInputRA6M3& operator=(const CGUIInputRA6M3& kSource);
    
private:
    TP_TouchFinger_st m_kLastEvent;

    eC_Int m_iTouchFilterDelta;
    
    TpEvt_EntryType m_eLastTouchEvent;
    
    eC_Int m_iLastPositionX;
    eC_Int m_iLastPositionY;

    eC_Int m_iLastDraggedX;
    eC_Int m_iLastDraggedY;
};
#endif
