#include "Menu.h"

#include "GUI.h"
#include "GUIAnimationAttribute.h"

#include "GUIGeometryObject.h"
#include "GUIIconButton.h"

#include "GUICallAPICmd.h"
#include "GUITransitionCmd.h"

#include "GfxEnv.h"
#include "GUIInputMedia.h"

#include "GUICommandHandler.h"

// LAST INCLUDE!!
#include "GUIMemLeakWatcher.h"

#define ICON_PADDING_X eC_FromInt(0)
#define ICON_PADDING_Y eC_FromInt(5)

eC_String MenuNames[] =
{
    "Animation",
    "Controls",
    "Advanced",
    "Data",
    "Container",
    "Keyboard",
    "Settings",
    "Playground",
    "Layer",
    "Extras"
};

ObjectHandle_t MenuObjects[] =
{
    DLG_ANIMATION,
    DLG_CONTROLS,
    DLG_ADVANCED,
    DLG_DATA,
    DLG_CONTAINER,
    DLG_KEYBOARD,
    DLG_SETTINGS,
    DLG_PLAYGROUND,
    DLG_LAYER,
    DLG_EXTRAS
};

ImageResource_t MenuIcons[] =
{
    IMG_ICON_DYNAMICS,
    IMG_ICON_CONTROLS,
    IMG_ICON_ADVANCED,
    IMG_ICON_DATA,
    IMG_ICON_CONTAINER,
    IMG_ICON_TEXT,
    IMG_ICON_SETTINGS,
    IMG_ICON_PLAYGROUND,
    IMG_ICON_LAYER,
    IMG_ICON_EXTRAS
};

DialogSwitch::DialogSwitch(Menu* pkMenu, const ObjectHandle_t& eID, const eC_String& kDestName) :
    CGUIBehaviourDecorator(),
    m_pkMenu(pkMenu),
    m_eID(eID),
    m_kDestName(kDestName)
{
}

eC_Bool DialogSwitch::DoClick(
    const eC_Value& vAbsX,
    const eC_Value& vAbsY)
{
    ObjectHandle_t eCurrentID = NO_HANDLE;
    for (eC_UInt uiChildIndex = 0; uiChildIndex < GETGUI.GetNumberOfChildren(); ++uiChildIndex)
    {
        if (GETGUI.GetChild(uiChildIndex)->GetID() != NO_HANDLE)
        {
            eCurrentID = GETGUI.GetChild(uiChildIndex)->GetID();
            break;
        }
    }
    if ((eCurrentID != NO_HANDLE) && (eCurrentID != m_eID))
    {
        if (NULL != m_pkMenu)
            m_pkMenu->ExpandCollapse();

        CGUICallAPICmd* pkCmd = new CGUICallAPICmd("Clean", "");
        pkCmd->AddAdditionalCmd(new CGUICallAPICmd("SetTransition", m_kDestName));
        pkCmd->AddAdditionalCmd(new CGUITransitionCmd(CGUITransitionCmd::BLEND_ONLY_SIMPLE, eCurrentID, m_kDestName, 10));
        pkCmd->AddAdditionalCmd(new CGUICallAPICmd(m_kDestName, ""));
        GETCMDHDL.Execute(pkCmd);
    }

    return true;
}

Menu::Menu(
    CGUICompositeObject* pkParent,
    const eC_Value& vMenuWidth, const eC_Value& vMenuHeight,
    const eC_Value& vButtonWidth, const eC_Value& vButtonHeight,
    const eC_Value& vIconSize) :
    CGUICompositeObject(pkParent, eC_FromInt(0), eC_FromInt(0), vMenuWidth, vButtonHeight),
    m_pkButton(NULL),
    m_pkButtonBox(NULL),
    m_bExpanded(false),
    m_vIconSize(vIconSize)
{
    // no events
    SetDisabled(true);

    // default icon count
    const eC_UInt cuiIconBaseCount = 8;
    eC_UInt uiIconCount = cuiIconBaseCount;
    if (GETGFXENV.GetNumberOfLayers() > 1)
        ++uiIconCount;
    if (GETINPUTMEDIA.GetMulitpleTouches())
        ++uiIconCount;

    eC_Value vMenuX = vButtonWidth;

    m_pkButton = new CGUIButton(this, eC_FromInt(0), eC_FromInt(0), vButtonWidth, vButtonHeight, DUMMY_TEXT, NULL);
    m_pkButton->SetImages(IMG_MENU_DOWN, IMG_MENU_DOWN, IMG_MENU_DOWN, IMG_MENU_DOWN, IMG_MENU_DOWN);
    m_pkButton->SetBehaviour(new MenuBehaviour(this));

    m_pkButtonBox = new CGUICompositeObject(this, vMenuX, -vMenuHeight, vMenuWidth, vMenuHeight);

    new CGUIGeometryObject(
        m_pkButtonBox,
        eC_FromInt(0), eC_FromInt(0), vMenuWidth, vMenuHeight,
        0xcccccccc, CGUIGeometryObject::PS_RECTANGLE, eC_FromInt(0), true);

    eC_Value vIconX = eC_Floor(eC_Div(vMenuWidth - vButtonWidth - (ICON_PADDING_X + eC_Mul(m_vIconSize + ICON_PADDING_X, eC_FromInt(uiIconCount))), eC_FromInt(2)));
    eC_Value vIconY = eC_FromInt(0);

    eC_UInt uiIconCountPerLine = uiIconCount;
    eC_Value vFreeSpace = vMenuWidth - vButtonWidth - ICON_PADDING_X;
    if (eC_Mul(eC_FromInt(uiIconCount), m_vIconSize + ICON_PADDING_X) > vFreeSpace)
    {
        uiIconCountPerLine = eC_ToInt(eC_Ceil(eC_Div(vFreeSpace, m_vIconSize + ICON_PADDING_X)));
        vIconX = ICON_PADDING_X;
    }

    eC_UInt uiMenuIndex;
    for (uiMenuIndex = 0; uiMenuIndex < cuiIconBaseCount; ++uiMenuIndex)
    {
        CreateMenuButton(uiMenuIndex, uiMenuIndex, uiIconCountPerLine, vIconX, vIconY);
    }

    if (GETGFXENV.GetNumberOfLayers() > 1)
    {
        CreateMenuButton(uiMenuIndex, 8, uiIconCountPerLine, vIconX, vIconY);
        ++uiMenuIndex;
    }

    if (GETINPUTMEDIA.GetMulitpleTouches())
    {
        CreateMenuButton(uiMenuIndex, 9, uiIconCountPerLine, vIconX, vIconY);
        ++uiMenuIndex;
    }
}

Menu::~Menu()
{
}

void Menu::CreateMenuButton(
    const eC_UInt& uiIndex, const eC_UInt& uiIconIndex,
    const eC_UInt& uiIconCountPerLine,
    const eC_Value& vIconX, const eC_Value& vIconY)
{
    eC_UInt uiMenuXIndex = uiIndex % uiIconCountPerLine;
    eC_UInt uiMenuYIndex = (uiIndex - uiMenuXIndex) / uiIconCountPerLine;
    eC_Value vCurrentX = vIconX + ICON_PADDING_X + eC_Mul(m_vIconSize + ICON_PADDING_X, eC_FromInt(uiMenuXIndex));
    eC_Value vCurrentY = vIconY + ICON_PADDING_Y + eC_Mul(m_vIconSize + ICON_PADDING_Y, eC_FromInt(uiMenuYIndex));

    CGUIButton* pkButton = new CGUIButton(
        m_pkButtonBox,
        vCurrentX, vCurrentY,
        m_vIconSize, m_vIconSize,
        DUMMY_TEXT,
        NULL);
    pkButton->SetImages(MenuIcons[uiIconIndex], MenuIcons[uiIconIndex], MenuIcons[uiIconIndex], MenuIcons[uiIconIndex], MenuIcons[uiIconIndex]);
    pkButton->SetBehaviour(new DialogSwitch(this, MenuObjects[uiIconIndex], MenuNames[uiIconIndex]));
}

void Menu::ExpandCollapse()
{
    CGUICompositeObject* pkParent = GetParent();
    if (NULL == pkParent)
        return;

    eC_Value vStart, vEnd;
    if (m_bExpanded)
    {
        vStart = eC_FromInt(0);
        vEnd = -m_pkButtonBox->GetHeight();
    }
    else
    {
        vStart = -m_pkButtonBox->GetHeight();
        vEnd = eC_FromInt(0);
    }

    CGUIEasing::EasingType_t eEasing = CGUIEasing::EASE_OUT_BOUNCE;
    if (m_bExpanded)
        eEasing = CGUIEasing::EASE_OUT_EXPO;

    CGUIAnimationAttribute* pkAnimation = new CGUIAnimationAttribute(
        m_pkButtonBox,
        500,
        CGUIAnimatedAttribute::ATTR_YPOS,
        eEasing,
        vStart,
        vEnd
    );
    pkAnimation->SetDeletedAfterFinish(true);
    pkAnimation->StartAnimation();

    m_bExpanded = !m_bExpanded;

    InvalidateArea();
    if (m_bExpanded)
    {
        m_pkButton->SetImages(IMG_MENU_UP, IMG_MENU_UP, IMG_MENU_UP, IMG_MENU_UP, IMG_MENU_UP);
        SetHeight(m_pkButtonBox->GetHeight());
    }
    else
    {
        m_pkButton->SetImages(IMG_MENU_DOWN, IMG_MENU_DOWN, IMG_MENU_DOWN, IMG_MENU_DOWN, IMG_MENU_DOWN);
        SetHeight(m_pkButton->GetHeight());
    }
    InvalidateArea();
}

MenuBehaviour::MenuBehaviour(Menu* pkMenu) :
    m_pkMenu(pkMenu)
{
}

eC_Bool MenuBehaviour::DoClick(
    const eC_Value& vAbsX,
    const eC_Value& vAbsY)
{
    if (NULL != m_pkMenu)
        m_pkMenu->ExpandCollapse();

    return true;
}
