/*
* Copyright (C) 2004 TES Electronic Solutions GmbH,
* All Rights Reserved.
* This source code and any compilation or derivative thereof is the
* proprietary information of TES Electronic Solutions GmbH
* and is confidential in nature.
* Under no circumstances is this software to be exposed to or placed
* under an Open Source License of any type without the expressed
* written permission of TES Electronic Solutions GmbH
*
*############################################################
*/

/******************************************************************************
 Includes   <System Includes> , "Project Includes"
 ******************************************************************************/

#include <main_thread.h>
#include <stdint.h>

#include "r_glcdc.h"

extern "C"
{
#include "m13_ra6m3_ek_led.h"
#include "m13_ra6m3_ek_vcom.h"
#include "m13_ra6m3_ek_adc.h"
#include "m13_ra6m3_ek_acc.h"
#include "m13_ra6m3_ek_ethernet.h"
#include "m13_ra6m3_ek_touch.h"
#include "m13_ra6m3_ek_lcd.h"
#include "FreeRTOS.h"

// framebuffer for D/AVE is placed in SDRAM
extern uint8_t lcd_fb_background [BSP_LCD_MAX_FRAME_BUFFER] [DISPLAY_BUFFER_STRIDE_BYTES_INPUT0 * 272];
// LCD-buffer is placed in SRAM
uint8_t screenbuffer[DISPLAY_BUFFER_STRIDE_BYTES_INPUT0 * 272] BSP_ALIGN_VARIABLE(64) BSP_PLACE_IN_SECTION(".bss");

extern glcdc_instance_ctrl_t g_display_ctrl;
extern const display_cfg_t g_display_cfg;

#define LCD_BYTES_PER_PIXEL (DISPLAY_BITS_PER_PIXEL_INPUT0 >> 3)

/******************************************************************************
 Private global variables and functions
 ******************************************************************************/

/******************************************************************************
 Imported global variables and functions (from other files)
 ******************************************************************************/

extern d2_device *d2_handle0;
d2_device** _d2_handle_guiliani = &d2_handle0;

volatile uint8_t *RZAFrameBuffers[] =
{
     lcd_fb_background[0],
     lcd_fb_background[0]
};

int fb_index = 0;
}

volatile uint8_t* GetFrameBuffer()
{
    return RZAFrameBuffers[fb_index];
}

uint32_t GetFrameBufferWidth()
{
	return LCD_WIDTH;
}

uint32_t GetFrameBufferHeight()
{
	return LCD_HEIGHT;
}

void FlipFramebuffer()
{
    R_GLCDC_BufferChange(
        &g_display_ctrl,
        (uint8_t*)screenbuffer,
        DISPLAY_FRAME_LAYER_1
    );

    vsync_wait();
}

void CopyBuffer(int iX1, int iY1, int iRectWidth, int iRectHeight)
{
    uint8_t* pubSource = lcd_fb_background[fb_index] + (iY1 * LCD_WIDTH + iX1) * LCD_BYTES_PER_PIXEL;
    uint8_t* pubDest = screenbuffer + (iY1 * LCD_WIDTH + iX1) * LCD_BYTES_PER_PIXEL;
    for (uint16_t iLineIndex = 0; iLineIndex < iRectHeight; ++iLineIndex)
    {
        memcpy(pubDest, pubSource, iRectWidth * LCD_BYTES_PER_PIXEL);

        pubSource += LCD_WIDTH * LCD_BYTES_PER_PIXEL;
        pubDest += LCD_WIDTH * LCD_BYTES_PER_PIXEL;
    }
}

void GraphicTest()
{
    uint8_t color = 0;

    while(1)
    {
        memset((void*)screenbuffer, color, LCD_WIDTH * LCD_HEIGHT * LCD_BYTES_PER_PIXEL);
        color += 0x12;

        FlipFramebuffer();
    }
}

void GraphicInit()
{
    bsp_lcd_io_init();

    fb_index = 0;

    memset((void*)screenbuffer, 0x3e, LCD_WIDTH * LCD_HEIGHT * LCD_BYTES_PER_PIXEL);
    memset((void*)RZAFrameBuffers[0], 0xff, LCD_WIDTH * LCD_HEIGHT * LCD_BYTES_PER_PIXEL);
    memset((void*)RZAFrameBuffers[1], 0x88, LCD_WIDTH * LCD_HEIGHT * LCD_BYTES_PER_PIXEL);

    bsp_lcd_init();
    bsp_lcd_display_on();

    bsp_lcd_set_backlight((bsp_io_level_t)1);

    bsp_lcd_enable_double_buffer();

    bsp_lcd_set_display_buffer(fb_index);

    R_GLCDC_BufferChange(
        &g_display_ctrl,
        (uint8_t*)screenbuffer,
        DISPLAY_FRAME_LAYER_1
    );
}

extern "C"
{
#if( configAPPLICATION_ALLOCATED_HEAP == 1 )
    extern volatile uint8_t ucHeap[ configTOTAL_HEAP_SIZE ];
#endif

void* GetHeapStart()
{
#if( configAPPLICATION_ALLOCATED_HEAP == 1 )
    return (void*)&ucHeap[0];
#else
    #error "no heap allocated"
    return NULL;
#endif
}

void* GetHeapEnd()
{
#if( configAPPLICATION_ALLOCATED_HEAP == 1 )
    return (void*)((uint8_t*)&ucHeap[0] + configTOTAL_HEAP_SIZE);
#else
    #error "no heap allocated"
    return NULL;
#endif
}

uint32_t GetHeapSize()
{
#if( configAPPLICATION_ALLOCATED_HEAP == 1 )
    return configTOTAL_HEAP_SIZE;
#else
    return 0;
#endif
}

void PrintString(const char* message)
{
    PRINTF_INFO(message);
}

void PrintChar(char c)
{
    
}
}

/*
 * QSPI_FLASH_START = 0x60000000;
 * QSPI_FLASH_LENGTH = 0x4000000;
 */
const long resource_size = 0x04000000;

extern "C"
{
// that's the name of the section in the linker-script
extern const uint8_t __attribute__ ((section (".qspi_flash"))) __qspi_flash_start__[resource_size];
const uint8_t* __guilianiResources = __qspi_flash_start__;
}

const uint8_t* GetResources()
{
    return __guilianiResources;
}
