#include "GUIInputRA6M3.h"

extern "C"
{
#include <ctype.h>
#include "FreeRTOS.h"
#include "task.h"
}

#define INVALID_AXIS_VALUE -1

#include "eC_String.h"
#include "GUITimer.h"
#include "GUITrace.h"
#include "GUIGestureDetector.h"

#include "GUIMemLeakWatcher.h"
#include "WindowsLeakWatcher.h"

#define DEBUG_TOUCH_EVENTS 0

static CGUIInputRA6M3* s_pkInputDevice = NULL;

extern "C" void touchevent_callback(TP_TouchFinger_st* psTouchEvt)
{
    if ((NULL != s_pkInputDevice) && (NULL != psTouchEvt))
    {
        s_pkInputDevice->AddEvent(*psTouchEvt);
    }
}

CGUIInputRA6M3::CGUIInputRA6M3(
    const eC_Int& iTouchScreenX,
    const eC_Int& iTouchScreenY,
    const eC_Int& iTouchScreenWidth,
    const eC_Int& iTouchScreenHeight,
    const eC_Int& iTouchFilterDelta) :
    m_iTouchFilterDelta(iTouchFilterDelta),
    m_iLastDraggedX(INVALID_AXIS_VALUE),
    m_iLastDraggedY(INVALID_AXIS_VALUE),
    m_iLastPositionX(INVALID_AXIS_VALUE),
    m_iLastPositionY(INVALID_AXIS_VALUE)
{
    s_pkInputDevice = this;

#ifdef GUILIANI_GESTURES_ACTIVE
    SetMultipleTouches(true);
#endif
}

CGUIInputRA6M3::~CGUIInputRA6M3()
{
}

void CGUIInputRA6M3::CreateInstance(
    const eC_Int& iTouchScreenX,
    const eC_Int& iTouchScreenY,
    const eC_Int& iTouchScreenWidth,
    const eC_Int& iTouchScreenHeight,
    const eC_Int& iTouchFilterDelta)
{
    if (!CGUIComponentManager::GetInstance().HasInputMedia())
        CGUIComponentManager::GetInstance().SetInputMedia(new CGUIInputRA6M3(iTouchScreenX, iTouchScreenY, iTouchScreenWidth, iTouchScreenHeight, iTouchFilterDelta));
}

void CGUIInputRA6M3::AddEvent(TP_TouchFinger_st kTouchEvent)
{
    CGUIEvent* pkEvent = NULL;

#ifdef GUILIANI_GESTURES_ACTIVE
    // save id of primary pointer
    static uint8_t primID = 0;
    if ((kTouchEvent.eState == TPEVT_ENTRY_DOWN) && (kTouchEvent.first_touch))
    {
        primID = kTouchEvent.id;
    }

    CGUIGestureDetector::PointerState_t pointer;
    pointer.m_iX = kTouchEvent.unPosX;
    pointer.m_iY = kTouchEvent.unPosY;
    pointer.m_uiPointerID = kTouchEvent.id;
    if (kTouchEvent.id == primID)
        pointer.m_bIsPrimary = true;
    else
        pointer.m_bIsPrimary = false;

    switch (kTouchEvent.eState)
    {
        case TPEVT_ENTRY_DOWN:
        {
            if (pointer.m_bIsPrimary)
                pkEvent = CGUIGestureDetector::ProcessEvent(CGUIGestureDetector::GET_PRIMARY_DOWN, pointer);
            else
                pkEvent = CGUIGestureDetector::ProcessEvent(CGUIGestureDetector::GET_ADDITIONAL_DOWN, pointer);
        }
        break;

        case TPEVT_ENTRY_UP:
        {
            if (pointer.m_bIsPrimary)
                pkEvent = CGUIGestureDetector::ProcessEvent(CGUIGestureDetector::GET_PRIMARY_UP, pointer);
            else
                pkEvent = CGUIGestureDetector::ProcessEvent(CGUIGestureDetector::GET_ADDITIONAL_UP, pointer);
        }
        break;

        case TPEVT_ENTRY_MOVE:
        {
            pkEvent = CGUIGestureDetector::ProcessEvent(CGUIGestureDetector::GET_MOVE, pointer);
        }
        break;
    }

    if (NULL != pkEvent)
    {
        CGUIInputMedia::AddEvent(pkEvent);
        return;
    }
#endif

    eC_Bool bCreateEvent = false;

    if (
        (m_eLastTouchEvent != kTouchEvent.eState) ||
        (abs(m_iLastPositionX - kTouchEvent.unPosX) > m_iTouchFilterDelta) ||
        (abs(m_iLastPositionY - kTouchEvent.unPosY) > m_iTouchFilterDelta)
        )
    {
        switch (kTouchEvent.eState)
        {
            case TPEVT_ENTRY_UP:
                m_iLastDraggedX = m_iLastDraggedY = INVALID_AXIS_VALUE;
                m_iLastPositionX = m_iLastPositionY = INVALID_AXIS_VALUE;
                break;

            case TPEVT_ENTRY_DOWN:
                m_iLastPositionX = kTouchEvent.unPosX;
                m_iLastPositionY = kTouchEvent.unPosY;
                break;

            case TPEVT_ENTRY_MOVE:
                m_iLastPositionX = kTouchEvent.unPosX;
                m_iLastPositionY = kTouchEvent.unPosY;
                break;

            default:
                break;
        }
        m_eLastTouchEvent = kTouchEvent.eState;
        bCreateEvent = true;
    }

    if (true == bCreateEvent)
    {
        CGUIEvent* pkEvent = NULL;
        switch (kTouchEvent.eState)
        {
            case TPEVT_ENTRY_UP:
                pkEvent = new CGUIMouseEvent(ET_LBUTTONUP, kTouchEvent.unPosX, kTouchEvent.unPosY);
                break;

            case TPEVT_ENTRY_DOWN:
                pkEvent = new CGUIMouseEvent(ET_LBUTTONDOWN, kTouchEvent.unPosX, kTouchEvent.unPosY, true);
                break;

            case TPEVT_ENTRY_MOVE:
                pkEvent = new CGUIMouseEvent(ET_MOUSEMOVE, kTouchEvent.unPosX, kTouchEvent.unPosY, true, false);
                break;

            default:
                break;
        }

        CGUIInputMedia::AddEvent(pkEvent);
    }
}
