/*
* Copyright (C) 2004 TES Electronic Solutions GmbH,
* All Rights Reserved.
* This source code and any compilation or derivative thereof is the
* proprietary information of TES Electronic Solutions GmbH
* and is confidential in nature.
* Under no circumstances is this software to be exposed to or placed
* under an Open Source License of any type without the expressed
* written permission of TES Electronic Solutions GmbH
*
*############################################################
*/

#include "StreamRuntimeStartup.h"
#include "Resource.h"

#include "GUITrace.h"
#include "GUIException.h"
#include "GUI.h"

#include "GfxEnvWin.h"

// Includes for wrapper classes
#include "SndWrapWin.h"
#include "GUIInputWindows.h"
#include "GUIPlaybackInput.h"

#if defined GFX_USE_EGML
    #include "GfxWrapeGML.h"
    #if defined FNT_USE_FREETYPE
        #include "FntWrapFreeType.h"
    #elif defined FNT_USE_GLYPHLIB
        #include "FntWrapGlyphLib.h"
    #endif
#elif defined GFX_USE_DAVE
    #include "GfxEnvWin.h"
    #include "GfxWrapDAVE.h"
    #if defined FNT_USE_FREETYPE
        #include "FntWrapFreeTypeDave.h"
    #elif defined FNT_USE_GLYPHLIB
        #include "FntWrapGlyphLibDave.h"
    #endif
#elif defined GFX_USE_DAVEHD
    #include "GfxWrapDaveHD.h"
    #if defined FNT_USE_FREETYPE
        #include "FntWrapFreeTypeDave.h"
    #elif defined FNT_USE_GLYPHLIB
        #include "FntWrapGlyphLibDave.h"
    #endif
#elif defined GFX_USE_GL
    #include "GfxWrapOGL.h"
    #if defined FNT_USE_FREETYPE
        #include "FntWrapFreeTypeGL.h"
    #elif defined FNT_USE_GLYPHLIB
        #include "FntWrapGlyphLibGL.h"
    #endif
#elif defined GFX_USE_BLU
    #include "GfxWrapBLU.h"
    #if defined FNT_USE_FREETYPE
        #include "FntWrapFreeType.h"
    #elif defined FNT_USE_GLYPHLIB
        #include "FntWrapGlyphLib.h"
    #endif
#elif defined GFX_USE_VULKAN
    #include "GfxWrapVulkan.h"
    #if defined FNT_USE_FREETYPE
        #include "FntWrapFreeType.h"
    #elif defined FNT_USE_GLYPHLIB
        #include "FntWrapGlyphLib.h"
    #endif
#else
    #include "GfxWrapNull.h"
#endif

#if !defined(GFX_USE_EGML)
#ifdef GUILIANI_ADD_IMAGE_DECODER
    #include "GUIImageLoader.h"
    #include "GUIImageDecoderBLU.h"
    #include "GUIImageDecoderBMP.h"
    #include "GUIImageDecoderPNG.h"
    #include "GUIImageDecoderJPG.h"
    #include "GUIImageDecoderRAW.h"
    #include "GUIImageDecoderRLE.h"
    #include "GUIImageDecoderSVG.h"
#else
#error "Guiliani is built without Imageloaders. Image will not be loaded correctly."
#endif
#endif

// needs to be included to avoid unresolved references to memleak
#include "GUIMemLeakDetection.h"

#include "GUIMemLeakWatcher.h"

/*
Startup sequence is:
    ConstructWrapperClasses()
    <--- Application lifetime --->
    DestructWrapperClasses()
*/

// Namespace containing default implementations for initializing a StreamRuntime under windows
namespace NStreamRuntime
{
    void CStreamRuntimeStartup::ConstructWrapperClasses()
    {
        CGUICommonEnums::ImageType_t eFormat = CGUICommonEnums::RGB_565;
#if defined GFX_USE_ARGB32
        eFormat = CGUICommonEnums::ARGB_8888;
#elif defined GFX_USE_RGB888
        eFormat = CGUICommonEnums::RGB_8888;
#elif defined GFX_USE_LUM8
        eFormat = CGUICommonEnums::LUM_8;
#endif

#if defined GFX_USE_GL
        CGfxEnvWin::CreateInstance(m_hInstance, CGfxEnv::SM_OPENGL, eFormat, GetScreenWidth(), GetScreenHeight());
#elif defined GFX_USE_DAVE
        CGfxEnvWin::CreateInstance(m_hInstance, CGfxEnv::SM_FRAMEBUFFER, eFormat, GetScreenWidth(), GetScreenHeight(), 2);
#elif defined GFX_USE_DAVEHD
        CGfxEnvWin::CreateInstance(m_hInstance, CGfxEnv::SM_FRAMEBUFFER, eFormat, GetScreenWidth(), GetScreenHeight(), 2);
#elif defined GFX_USE_VULKAN
        CGfxEnvWin::CreateInstance(m_hInstance, CGfxEnv::SM_VULKAN, eFormat, GetScreenWidth(), GetScreenHeight());
#else
        CGfxEnvWin::CreateInstance(m_hInstance, CGfxEnv::SM_FRAMEBUFFER, eFormat, GetScreenWidth(), GetScreenHeight());
#endif

        // initialize layers from configuration
        const eC_UInt uiLayerCount = GETRUNTIMECONFIG.GetLayerCount();
        for (eC_UInt uiLayerIndex = 0; uiLayerIndex < uiLayerCount; ++uiLayerIndex)
        {
            GETGFXENV.AddLayer(GETRUNTIMECONFIG.GetLayerConfig(uiLayerIndex));
        }
        GETGFXENV.InitLayers();

// Sets the graphics wrapper and font wrapper
#if defined GFX_USE_EGML
    #if defined GFX_USE_DCWRAP
        CGfxWrapeGML::CreateInstance(&GETGFXENV);
    #else
        CGfxWrapeGML::CreateInstance(&GETGFXENV);
    #endif

    #if defined FNT_USE_FREETYPE
            CFntWrapFreeType::CreateInstance(128 * 128);
    #elif defined FNT_USE_GLYPHLIB
            CFntWrapGlyphLib::CreateInstance();
    #endif
#elif defined GFX_USE_GL
        CGfxWrapOGL::CreateInstance(&GETGFXENV);
#if defined FNT_USE_FREETYPE
        CFntWrapFreeTypeGL::CreateInstance();
#elif defined FNT_USE_GLYPHLIB
        CFntWrapGlyphLibGL::CreateInstance();
#endif
#elif defined GFX_USE_DAVE
        CGfxWrapDAVE::CreateInstance(&GETGFXENV, 80000);
#if defined FNT_USE_FREETYPE
        CFntWrapFreeTypeDAVE::CreateInstance(1024 * 1024);
#elif defined FNT_USE_GLYPHLIB
        CFntWrapGlyphLibDAVE::CreateInstance();
#endif
#elif defined GFX_USE_DAVEHD
        CGfxWrapDAVEHD::CreateInstance(&GETGFXENV);
#if defined FNT_USE_FREETYPE
        CFntWrapFreeTypeDAVE::CreateInstance(1024 * 1024);
#elif defined FNT_USE_GLYPHLIB
        CFntWrapGlyphLibDAVE::CreateInstance();
#endif
#elif defined GFX_USE_BLU
        CGfxWrapBLU::CreateInstance(&GETGFXENV);
#if defined FNT_USE_FREETYPE
        CFntWrapFreeType::CreateInstance(1024 * 1024);
#elif defined FNT_USE_GLYPHLIB
        CFntWrapGlyphLib::CreateInstance();
#endif
#elif defined GFX_USE_VULKAN
        CGfxWrapVulkan::CreateInstance(&GETGFXENV);
#if defined FNT_USE_FREETYPE
        CFntWrapFreeType::CreateInstance(1024 * 1024);
#elif defined FNT_USE_GLYPHLIB
        CFntWrapGlyphLib::CreateInstance();
#endif
#else
        MessageBox(NULL, TEXT("Please choose a Graphics-Wrapper. Defaulting to CGfxWrapNull."), TEXT("Warning"), 0);
        CGfxWrapNull::CreateInstance();
#endif

#if !defined(GFX_USE_EGML)
    #ifdef GUILIANI_ADD_IMAGE_DECODER
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderBMP());
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderBLU());
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderPNG());
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderJPG());
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderRAW());
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderRLE());
            GETIMGLOADER.AddDecoder(new CGUIImageDecoderSVG());
    #endif
#endif

        // Sets the sound wrapper
        CSndWrapWin::CreateInstance();
        // Sets the input wrapper
        CGUIInputWindows::CreateInstance((HWND)GETGFXENV.GetWindowHandle(), false);
    }

    void CStreamRuntimeStartup::Init(HINSTANCE hInstance, int nCmdShow)
    {
        m_hInstance = hInstance;
    }

}
