/****************************************************************************
* copyright:       TES Electronic Solutions GmbH
* project:         eC
* file name:       eC_x86_win32_Platform.h
* %version:        134.0 %
* 
* author:          hh04005
* creation date:   Thu May 04 17:08:20 2006
* description:

* %date_modified:  % (%derived_by: hh04005 %)
*
* history:
* date         | name          | description of modifications
*-------------------------------------------------------------------------
  05-08-03     | hh04031       | Exchange fileheader with CM Synergy-Tags

******************************************************************************/

//
// Interface to platform specific functions
//

#ifndef EC_X86_WIN32_PLATFORM__H_
#define EC_X86_WIN32_PLATFORM__H_
//---------------------------------------------------------------
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include "eC_Types.h"


// platform API unicode encoding
#define eC_STRING_UNICODE_UTF16

//---------------------------------------------------------------
//Eliminate dependencies to definitions within the windows-library
#ifdef eC_TARGET_ENV_WIN32
#ifndef CINTERFACE
#define CINTERFACE
#endif
#endif


//---------------------------------------------------------------
// basic clib functions
extern "C"
{
void* eC_Malloc(size_t tSize);
void* eC_Calloc(size_t n, size_t tSize);
void* eC_Realloc(void* pData, size_t tNewSize);
void eC_Free(void* pData);
void* eC_Memcpy(void* dest, const void* src, size_t count);
void* eC_Memmove(void* dest, const void* src, size_t count);
void* eC_Memset(void* dest, int c, size_t count);

void eC_SetAvailableMemory(const size_t& uiAvailable);
size_t eC_GetAvailableMemory();
size_t eC_GetUsedMemory();
size_t eC_GetUsedMemoryMax();
size_t eC_GetUsedMemoryLargestBlock();
size_t eC_GetFreeMemory();
size_t eC_GetNumAllocs();

void eC_ClearUsedMemoryMax();
void eC_ClearUsedMemoryLargestBlock();
}

//---------------------------------------------------------------
// basic fileio functions

#define eC_SEEK_BEGIN   SEEK_SET
#define eC_SEEK_CURRENT SEEK_CUR
#define eC_SEEK_END     SEEK_END

class eC_PlatformFileIO
{
public:
    typedef struct _FILE_FIND_DATA
    {
        eC_WChar pwcFileName[256];
    } FILE_FIND_DATA;

    static void* eC_PlatformFOpen(const eC_Char *acName, const char *acMode);
    static void* eC_PlatformFOpen(const eC_WChar *acName, const eC_WChar *acMode);
    static void eC_PlatformFClose(void* pvFile);
    static eC_Bool eC_PlatformFSeek(void* pvFile, eC_Int iOffset, eC_UInt uiFlag);
    static eC_UInt eC_PlatformFTell(void* pvFile);
    static eC_UInt eC_PlatformFWrite(void* pvFile, void* pvData, eC_UInt uiSize);
    static eC_UInt eC_PlatformFRead(void* pvFile, void* pvData, eC_UInt uiSize);
    static void* eC_PlatformFindFirstFile(const eC_WChar *acName, FILE_FIND_DATA& fileData);
    static eC_Bool eC_PlatformFindNextFile(void* pvFile, FILE_FIND_DATA& fileData);
    static eC_Bool eC_PlatformFindClose(void* pvFile);
};

//---------------------------------------------------------------
// debug interface

#define eC_GETTICKS() eC_GetTicks64()
#define eC_GET_MICROSECONDS(tTime) eC_GetMicroSec(tTime)

void eC_PlatformStringOut(const char *acMessage);
void eC_PlatformCharOut(char c);

eC_Time64 eC_GetTicks64();
eC_UInt eC_GetMS(eC_Time64 tTime);
eC_UInt eC_GetMicroSec(eC_Time64 tTime);

//---------------------------------------------------------------
// task/thread suspend/switch interface
eC_Bool eC_Wait(eC_UInt ms, eC_Flag bActive = false);

#endif
